#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""
motor.py: 
"""


from taurus.external.qt import Qt
from taurus.qt.qtgui.base import TaurusBaseWidget
from taurus.qt.qtgui.util.ui import UILoadable


def showDialogConfigureMotor(parent):
    Dialog = Qt.QDialog(parent)
    Dialog.resize((Qt.QSize(Qt.QRect(0, 0, 310, 309).size()).expandedTo(Dialog.minimumSizeHint())))
    motorV2 = TaurusMotorV2(Dialog)
    motorV2.setModel(parent.model)
    motorV2.setGeometry(Qt.QRect(10, 10, 291, 291))
    Dialog.show()


@UILoadable(with_ui='ui')
class TaurusMotorH(Qt.QWidget, TaurusBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent=None, designMode=False):
        self.call__init__wo_kw(Qt.QWidget, parent)
        self.call__init__(TaurusBaseWidget, str(self.objectName()), designMode=designMode)
        self.loadUi()
        Qt.QObject.connect(self.ui.config, Qt.SIGNAL("clicked()"), self.configureMotor)

    def sizeHint(self):
        return Qt.QSize(330, 50)

    def configureMotor(self):
        showDialogConfigureMotor(self.ui.TaurusGroupBox)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return None
#        ret = TaurusBaseWidget.getQtDesignerPluginInfo()
#        ret['module'] = 'taurus.qt.qtgui.extra_pool'
#        ret['group'] = 'Taurus Sardana'
#        ret['icon'] = ':/designer/extra_pool.png'
#        return ret

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @Qt.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TaurusGroupBox.getModel()

    @Qt.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TaurusGroupBox.setModel(model)

    @Qt.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TaurusGroupBox.resetModel()


    @Qt.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TaurusGroupBox.getShowText()

    @Qt.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TaurusGroupBox.setShowText(showText)

    @Qt.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TaurusGroupBox.resetShowText()

    model = Qt.pyqtProperty("QString", getModel, setModel, resetModel)
    showText = Qt.pyqtProperty("bool", getShowText, setShowText, resetShowText)


@UILoadable(with_ui='ui')
class TaurusMotorH2(Qt.QWidget, TaurusBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent=None, designMode=False):
        self.call__init__wo_kw(Qt.QWidget, parent)
        self.call__init__(TaurusBaseWidget, str(self.objectName()), designMode=designMode)
        self.loadUi()
        Qt.QObject.connect(self.ui.config, Qt.SIGNAL("clicked()"), self.configureMotor)

    def sizeHint(self):
        return Qt.QSize(215, 85)

    def configureMotor(self):
        showDialogConfigureMotor(self.ui.TaurusGroupBox)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return None
#        ret = TaurusBaseWidget.getQtDesignerPluginInfo()
#        ret['module'] = 'taurus.qt.qtgui.extra_pool'
#        ret['group'] = 'Taurus Sardana'
#        ret['icon'] = ':/designer/extra_pool.png'
#        return ret

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @Qt.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TaurusGroupBox.getModel()

    @Qt.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TaurusGroupBox.setModel(model)

    @Qt.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TaurusGroupBox.resetModel()


    @Qt.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TaurusGroupBox.getShowText()

    @Qt.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TaurusGroupBox.setShowText(showText)

    @Qt.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TaurusGroupBox.resetShowText()


    model = Qt.pyqtProperty("QString", getModel, setModel, resetModel)
    showText = Qt.pyqtProperty("bool", getShowText, setShowText, resetShowText)


@UILoadable(with_ui='ui')
class TaurusMotorV(Qt.QWidget, TaurusBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent=None, designMode=False):
        self.call__init__wo_kw(Qt.QWidget, parent)
        self.call__init__(TaurusBaseWidget, str(self.objectName()), designMode=designMode)
        self.loadUi()        
        Qt.QObject.connect(self.ui.config, Qt.SIGNAL("clicked()"), self.configureMotor)

    def sizeHint(self):
        return Qt.QSize(120, 145)

    def configureMotor(self):
        showDialogConfigureMotor(self.ui.TaurusGroupBox)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return None
#        ret = TaurusBaseWidget.getQtDesignerPluginInfo()
#        ret['module'] = 'taurus.qt.qtgui.extra_pool'
#        ret['group'] = 'Taurus Sardana'
#        ret['icon'] = ':/designer/extra_pool.png'
#        return ret

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @Qt.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TaurusGroupBox.getModel()

    @Qt.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TaurusGroupBox.setModel(model)

    @Qt.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TaurusGroupBox.resetModel()


    @Qt.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TaurusGroupBox.getShowText()

    @Qt.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TaurusGroupBox.setShowText(showText)

    @Qt.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TaurusGroupBox.resetShowText()


    model = Qt.pyqtProperty("QString", getModel, setModel, resetModel)
    showText = Qt.pyqtProperty("bool", getShowText, setShowText, resetShowText)


@UILoadable(with_ui='ui')
class TaurusMotorV2(Qt.QWidget, TaurusBaseWidget):

    __pyqtSignals__ = ("modelChanged(const QString &)",)

    def __init__(self, parent=None, designMode=False):
        self.call__init__wo_kw(Qt.QWidget, parent)
        self.call__init__(TaurusBaseWidget, str(self.objectName()), designMode=designMode)
        self.loadUi()

    def sizeHint(self):
        return Qt.QSize(300, 275)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return None
#        ret = TaurusBaseWidget.getQtDesignerPluginInfo()
#        ret['module'] = 'taurus.qt.qtgui.extra_pool'
#        ret['group'] = 'Taurus Sardana'
#        ret['icon'] = ':/designer/extra_pool.png'
#        return ret

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # QT properties
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    @Qt.pyqtSignature("getModel()")
    def getModel(self):
        return self.ui.TaurusGroupBox.getModel()

    @Qt.pyqtSignature("setModel(QString)")
    def setModel(self, model):
        self.ui.TaurusGroupBox.setModel(model)

    @Qt.pyqtSignature("resetModel()")
    def resetModel(self):
        self.ui.TaurusGroupBox.resetModel()


    @Qt.pyqtSignature("getShowText()")
    def getShowText(self):
        return self.ui.TaurusGroupBox.getShowText()

    @Qt.pyqtSignature("setShowText(bool)")
    def setShowText(self, showText):
        self.ui.TaurusGroupBox.setShowText(showText)

    @Qt.pyqtSignature("resetShowText()")
    def resetShowText(self):
        self.ui.TaurusGroupBox.resetShowText()


    model = Qt.pyqtProperty("QString", getModel, setModel, resetModel)
    showText = Qt.pyqtProperty("bool", getShowText, setShowText, resetShowText)


###class TaurusMotorH2(Qt.QGroupBox, TaurusBaseWidget):
###
###    def __init__(self, parent = None, designMode = False):
###        name = "TaurusMotorH2"
###        self._prefix = ''
###        self._suffix = ''
###
###        self.call__init__wo_kw(Qt.QGroupBox, parent)
###        self.call__init__(TaurusBaseWidget, name, designMode = designMode)
###
###        self.setObjectName(name)
###        self.defineStyle()
###
###
###        ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###        ## SO ALL THE STUFFABOVE IS NECESSARY
###        ##
###        ## ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
###        ##############################################################################
###        ##
###        #class TaurusMotorH2(TaurusGroupBox):
###        #def __init__(self, parent = None):
###        #self.call__init__wo_kw(TaurusGroupBox,parent)
###        #self.call__init__(TaurusGroupBox,str(self.objectName()))
###        self.setupUi()
###        self.retranslateUi()
###        self.connect(self.config,Qt.SIGNAL('clicked()'),self.configureMotor)
###
###
###    def configureMotor(self):
###        Dialog = Qt.QDialog(self)
###        Dialog.resize((Qt.QSize(Qt.QRect(0,0,310,309).size()).expandedTo(Dialog.minimumSizeHint())))
###        motorV2 = TaurusMotorV2(Dialog)
###        motorV2.setModel(self.model)
###        motorV2.setGeometry(Qt.QRect(10,10,291,291))
###        Dialog.show()
###
###
###
###    def minimumSizeHint(self):
###        return Qt.QSize(211,80)
###
###    def sizeHint(self):
###        return Qt.QSize(211,80)
###
###
###    def setupUi(self):
###
###        self.gridlayout = Qt.QGridLayout(self)
###        self.gridlayout.setObjectName("gridlayout")
###
###        self.vboxlayout = Qt.QVBoxLayout()
###        self.vboxlayout.setObjectName("vboxlayout")
###
###        self.hboxlayout = Qt.QHBoxLayout()
###        self.hboxlayout.setObjectName("hboxlayout")
###
###        self.taurusValueLabel_2 = TaurusValueLabel(self)
###        self.taurusValueLabel_2.setFrameShape(Qt.QFrame.NoFrame)
###        self.taurusValueLabel_2.setFrameShadow(Qt.QFrame.Plain)
###        self.taurusValueLabel_2.setShowQuality(False)
###        self.taurusValueLabel_2.setUseParentModel(True)
###        self.taurusValueLabel_2.setObjectName("taurusValueLabel_2")
###        self.hboxlayout.addWidget(self.taurusValueLabel_2)
###
###        self.TaurusStateLed_17 = TaurusStateLed(self)
###        self.TaurusStateLed_17.setUseParentModel(True)
###        self.TaurusStateLed_17.setObjectName("TaurusStateLed_17")
###        self.hboxlayout.addWidget(self.TaurusStateLed_17)
###
###        spacerItem = Qt.QSpacerItem(40,20,Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Minimum)
###        self.hboxlayout.addItem(spacerItem)
###
###        self.TaurusLimitSwitch = TaurusLimitSwitch(self)
###        self.TaurusLimitSwitch.setUseParentModel(True)
###        self.TaurusLimitSwitch.setBoolIndex(2)
###        self.TaurusLimitSwitch.setObjectName("TaurusLimitSwitch")
###        self.hboxlayout.addWidget(self.TaurusLimitSwitch)
###
###        self.label_3 = Qt.QLabel(self)
###        self.label_3.setAlignment(Qt.Qt.AlignCenter)
###        self.label_3.setObjectName("label_3")
###        self.hboxlayout.addWidget(self.label_3)
###
###        self.TaurusLimitSwitch_2 = TaurusLimitSwitch(self)
###        self.TaurusLimitSwitch_2.setUseParentModel(True)
###        self.TaurusLimitSwitch_2.setBoolIndex(1)
###        self.TaurusLimitSwitch_2.setObjectName("TaurusLimitSwitch_2")
###        self.hboxlayout.addWidget(self.TaurusLimitSwitch_2)
###        self.vboxlayout.addLayout(self.hboxlayout)
###
###        self.hboxlayout1 = Qt.QHBoxLayout()
###        self.hboxlayout1.setObjectName("hboxlayout1")
###
###        self.TaurusValueLineEdit_21 = TaurusValueLineEdit(self)
###        self.TaurusValueLineEdit_21.setUseParentModel(True)
###        self.TaurusValueLineEdit_21.setObjectName("TaurusValueLineEdit_21")
###        self.hboxlayout1.addWidget(self.TaurusValueLineEdit_21)
###
###        self.taurusValueLabel_21 = TaurusValueLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.taurusValueLabel_21.sizePolicy().hasHeightForWidth())
###        self.taurusValueLabel_21.setSizePolicy(sizePolicy)
###        self.taurusValueLabel_21.setUseParentModel(True)
###        self.taurusValueLabel_21.setObjectName("taurusValueLabel_21")
###        self.hboxlayout1.addWidget(self.taurusValueLabel_21)
###
###        self.taurusConfigLabel_18 = TaurusConfigLabel(self)
###        self.taurusConfigLabel_18.setMaximumSize(Qt.QSize(27,22))
###        self.taurusConfigLabel_18.setUseParentModel(True)
###        self.taurusConfigLabel_18.setObjectName("taurusConfigLabel_18")
###        self.hboxlayout1.addWidget(self.taurusConfigLabel_18)
###
###        self.config = Qt.QToolButton(self)
###        self.config.setObjectName("config")
###        self.hboxlayout1.addWidget(self.config)
###        self.vboxlayout.addLayout(self.hboxlayout1)
###        self.gridlayout.addLayout(self.vboxlayout,0,0,1,1)
###
###
###    def retranslateUi(self):
###        self.taurusValueLabel_2.setModel(Qt.QApplication.translate("Form", "/State", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusStateLed_17.setModel(Qt.QApplication.translate("Form", "/State", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch.setModel(Qt.QApplication.translate("Form", "/Limit_switches", None, Qt.QApplication.UnicodeUTF8))
###        self.label_3.setText(Qt.QApplication.translate("Form", "- lim +", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch_2.setModel(Qt.QApplication.translate("Form", "/Limit_switches", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusValueLineEdit_21.setModel(Qt.QApplication.translate("Form", "/Position", None, Qt.QApplication.UnicodeUTF8))
###        self.taurusValueLabel_21.setModel(Qt.QApplication.translate("Form", "/Position", None, Qt.QApplication.UnicodeUTF8))
###        self.taurusConfigLabel_18.setModel(Qt.QApplication.translate("Form", "/Position?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.config.setText(Qt.QApplication.translate("Form", "cfg", None, Qt.QApplication.UnicodeUTF8))
###
###
###    ##############################################################################
###    ## VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
###    ##
###    ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###    ## SO ALL THE STUFF BELOW IS NECESSARY
###
###
###
###    def defineStyle(self):
###        palette = Qt.QPalette()
###        self.setPalette(palette)
###        self.updateStyle()
###
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # TaurusBaseWidget over writing
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###
###    def getDisplayValue(self):
###        return (self._prefix or '') + TaurusBaseWidget.getDisplayValue(self) + (self._suffix or '')
###
###    def isReadOnly(self):
###        return True
###
###    def updateStyle(self):
###        if self.getShowQuality():
###            self.setAutoFillBackground(True)
###            #TODO: get quality/state from model and update accordingly
###        else:
###            self.setAutoFillBackground(False)
###            #TODO: restore colors
###        self.update()
###
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # QT properties
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###
###    def getPrefixText(self):
###        return self._prefix
###
###    @Qt.pyqtSignature("setPrefixText(QString)")
###    def setPrefixText(self,prefix):
###        self._prefix = prefix
###        self.fireValueChanged()
###
###    def getSuffixText(self):
###        return self._suffix
###
###    @Qt.pyqtSignature("setSuffixText(QString)")
###    def setSuffixText(self,suffix):
###        self._suffix = suffix
###        self.fireValueChanged()
###
###    model = Qt.pyqtProperty("QString", TaurusBaseWidget.getModel,
###                                TaurusBaseWidget.setModel, TaurusBaseWidget.resetModel)
###    useParentModel = Qt.pyqtProperty("bool", TaurusBaseWidget.getUseParentModel,
###                                         TaurusBaseWidget.setUseParentModel,
###                                         TaurusBaseWidget.resetUseParentModel)
###    showQuality = Qt.pyqtProperty("bool", TaurusBaseWidget.getShowQuality,
###                                      TaurusBaseWidget.setShowQuality,
###                                      TaurusBaseWidget.resetShowQuality)
###    showText = Qt.pyqtProperty("bool", TaurusBaseWidget.getShowText,
###                                   TaurusBaseWidget.setShowText,
###                                   TaurusBaseWidget.resetShowText)
###    prefixText = Qt.pyqtProperty("QString", getPrefixText, setPrefixText,
###                                     doc="prefix text (optional)")
###    suffixText = Qt.pyqtProperty("QString", getSuffixText, setSuffixText,
###                                    doc="suffix text (optional)")
###
###
###
###
###class TaurusMotorV(Qt.QGroupBox, TaurusBaseWidget):
###
###    def __init__(self, parent = None):
###        name = "TaurusMotorV"
###        self._prefix = ''
###        self._suffix = ''
###
###        self.call__init__wo_kw(Qt.QGroupBox, parent)
###        self.call__init__(TaurusBaseWidget, name)
###
###        self.setObjectName(name)
###        self.defineStyle()
###
###
###        ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###        ## SO ALL THE STUFFABOVE IS NECESSARY
###        ##
###        ## ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
###        ##############################################################################
###        ##
###        #class TaurusMotorV(TaurusGroupBox):
###        #def __init__(self, parent = None):
###        #self.call__init__wo_kw(TaurusGroupBox,parent)
###        #self.call__init__(TaurusGroupBox,str(self.objectName()))
###        self.setupUi()
###        self.retranslateUi()
###        self.connect(self.config,Qt.SIGNAL('clicked()'),self.configureMotor)
###
###
###    def configureMotor(self):
###        Dialog = Qt.QDialog(self)
###        Dialog.resize((Qt.QSize(Qt.QRect(0,0,310,309).size()).expandedTo(Dialog.minimumSizeHint())))
###        motorV2 = TaurusMotorV2(Dialog)
###        motorV2.setModel(self.model)
###        motorV2.setGeometry(Qt.QRect(10,10,291,291))
###        Dialog.show()
###
###
###
###
###    def minimumSizeHint(self):
###        return Qt.QSize(150,128)
###
###    def sizeHint(self):
###        return Qt.QSize(150,128)
###
###    def setupUi(self):
###
###        self.gridlayout = Qt.QGridLayout(self)
###        self.gridlayout.setObjectName("gridlayout")
###
###        self.vboxlayout = Qt.QVBoxLayout()
###        self.vboxlayout.setObjectName("vboxlayout")
###
###        self.hboxlayout = Qt.QHBoxLayout()
###        self.hboxlayout.setObjectName("hboxlayout")
###
###        self.taurusValueLabel_2 = TaurusValueLabel(self)
###        self.taurusValueLabel_2.setFrameShape(Qt.QFrame.NoFrame)
###        self.taurusValueLabel_2.setFrameShadow(Qt.QFrame.Plain)
###        self.taurusValueLabel_2.setShowQuality(False)
###        self.taurusValueLabel_2.setUseParentModel(True)
###        self.taurusValueLabel_2.setObjectName("taurusValueLabel_2")
###        self.hboxlayout.addWidget(self.taurusValueLabel_2)
###
###        self.TaurusStateLed_17 = TaurusStateLed(self)
###        self.TaurusStateLed_17.setUseParentModel(True)
###        self.TaurusStateLed_17.setObjectName("TaurusStateLed_17")
###        self.hboxlayout.addWidget(self.TaurusStateLed_17)
###        self.vboxlayout.addLayout(self.hboxlayout)
###
###        self.hboxlayout1 = Qt.QHBoxLayout()
###        self.hboxlayout1.setObjectName("hboxlayout1")
###
###        self.TaurusValueLineEdit_21 = TaurusValueLineEdit(self)
###        self.TaurusValueLineEdit_21.setUseParentModel(True)
###        self.TaurusValueLineEdit_21.setObjectName("TaurusValueLineEdit_21")
###        self.hboxlayout1.addWidget(self.TaurusValueLineEdit_21)
###
###        self.config = Qt.QToolButton(self)
###        self.config.setObjectName("config")
###        self.hboxlayout1.addWidget(self.config)
###        self.vboxlayout.addLayout(self.hboxlayout1)
###
###        self.hboxlayout2 = Qt.QHBoxLayout()
###        self.hboxlayout2.setObjectName("hboxlayout2")
###
###        self.taurusValueLabel_21 = TaurusValueLabel(self)
###        self.taurusValueLabel_21.setUseParentModel(True)
###        self.taurusValueLabel_21.setObjectName("taurusValueLabel_21")
###        self.hboxlayout2.addWidget(self.taurusValueLabel_21)
###
###        self.taurusConfigLabel_18 = TaurusConfigLabel(self)
###        self.taurusConfigLabel_18.setMaximumSize(Qt.QSize(27,22))
###        self.taurusConfigLabel_18.setUseParentModel(True)
###        self.taurusConfigLabel_18.setObjectName("taurusConfigLabel_18")
###        self.hboxlayout2.addWidget(self.taurusConfigLabel_18)
###        self.vboxlayout.addLayout(self.hboxlayout2)
###
###        self.hboxlayout3 = Qt.QHBoxLayout()
###        self.hboxlayout3.setObjectName("hboxlayout3")
###
###        self.TaurusLimitSwitch = TaurusLimitSwitch(self)
###        self.TaurusLimitSwitch.setUseParentModel(True)
###        self.TaurusLimitSwitch.setBoolIndex(2)
###        self.TaurusLimitSwitch.setObjectName("TaurusLimitSwitch")
###        self.hboxlayout3.addWidget(self.TaurusLimitSwitch)
###
###        self.label_3 = Qt.QLabel(self)
###        self.label_3.setAlignment(Qt.Qt.AlignCenter)
###        self.label_3.setObjectName("label_3")
###        self.hboxlayout3.addWidget(self.label_3)
###
###        self.TaurusLimitSwitch_2 = TaurusLimitSwitch(self)
###        self.TaurusLimitSwitch_2.setUseParentModel(True)
###        self.TaurusLimitSwitch_2.setBoolIndex(1)
###        self.TaurusLimitSwitch_2.setObjectName("TaurusLimitSwitch_2")
###        self.hboxlayout3.addWidget(self.TaurusLimitSwitch_2)
###        self.vboxlayout.addLayout(self.hboxlayout3)
###        self.gridlayout.addLayout(self.vboxlayout,0,0,1,1)
###
###
###    def retranslateUi(self):
###        self.taurusValueLabel_2.setModel(Qt.QApplication.translate("Form", "/State", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusStateLed_17.setModel(Qt.QApplication.translate("Form", "/State", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusValueLineEdit_21.setModel(Qt.QApplication.translate("Form", "/Position", None, Qt.QApplication.UnicodeUTF8))
###        self.config.setText(Qt.QApplication.translate("Form", "cfg", None, Qt.QApplication.UnicodeUTF8))
###        self.taurusValueLabel_21.setModel(Qt.QApplication.translate("Form", "/Position", None, Qt.QApplication.UnicodeUTF8))
###        self.taurusConfigLabel_18.setModel(Qt.QApplication.translate("Form", "/Position?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch.setModel(Qt.QApplication.translate("Form", "/Limit_switches", None, Qt.QApplication.UnicodeUTF8))
###        self.label_3.setText(Qt.QApplication.translate("Form", "- lim +", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch_2.setModel(Qt.QApplication.translate("Form", "/Limit_switches", None, Qt.QApplication.UnicodeUTF8))
###
###    ##############################################################################
###    ## VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
###    ##
###    ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###    ## SO ALL THE STUFF BELOW IS NECESSARY
###
###
###
###    def defineStyle(self):
###        palette = Qt.QPalette()
###        self.setPalette(palette)
###        self.updateStyle()
###
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # TaurusBaseWidget over writing
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###
###    def getDisplayValue(self):
###        return (self._prefix or '') + TaurusBaseWidget.getDisplayValue(self) + (self._suffix or '')
###
###    def isReadOnly(self):
###        return True
###
###    def updateStyle(self):
###        if self.getShowQuality():
###            self.setAutoFillBackground(True)
###            #TODO: get quality/state from model and update accordingly
###        else:
###            self.setAutoFillBackground(False)
###            #TODO: restore colors
###        self.update()
###
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # QT properties
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###
###    def getPrefixText(self):
###        return self._prefix
###
###    @Qt.pyqtSignature("setPrefixText(QString)")
###    def setPrefixText(self,prefix):
###        self._prefix = prefix
###        self.fireValueChanged()
###
###    def getSuffixText(self):
###        return self._suffix
###
###    @Qt.pyqtSignature("setSuffixText(QString)")
###    def setSuffixText(self,suffix):
###        self._suffix = suffix
###        self.fireValueChanged()
###
###    model = Qt.pyqtProperty("QString", TaurusBaseWidget.getModel,
###                                TaurusBaseWidget.setModel, TaurusBaseWidget.resetModel)
###    useParentModel = Qt.pyqtProperty("bool", TaurusBaseWidget.getUseParentModel,
###                                         TaurusBaseWidget.setUseParentModel,
###                                         TaurusBaseWidget.resetUseParentModel)
###    showQuality = Qt.pyqtProperty("bool", TaurusBaseWidget.getShowQuality,
###                                      TaurusBaseWidget.setShowQuality,
###                                      TaurusBaseWidget.resetShowQuality)
###    showText = Qt.pyqtProperty("bool", TaurusBaseWidget.getShowText,
###                                   TaurusBaseWidget.setShowText,
###                                   TaurusBaseWidget.resetShowText)
###    prefixText = Qt.pyqtProperty("QString", getPrefixText, setPrefixText,
###                                     doc="prefix text (optional)")
###    suffixText = Qt.pyqtProperty("QString", getSuffixText, setSuffixText,
###                                    doc="suffix text (optional)")
###
###
###
###class TaurusMotorV2(Qt.QGroupBox, TaurusBaseWidget):
###
###    def __init__(self, parent = None):
###        name = "TaurusMotorV2"
###        self._prefix = ''
###        self._suffix = ''
###
###        self.call__init__wo_kw(Qt.QGroupBox, parent)
###        self.call__init__(TaurusBaseWidget, name)
###
###        self.setObjectName(name)
###        self.defineStyle()
###
###
###        ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###        ## SO ALL THE STUFFABOVE IS NECESSARY
###        ##
###        ## ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
###        ##############################################################################
###        ##
###        #class TaurusMotorV2(TaurusGroupBox):
###        #def __init__(self, parent = None):
###        #self.call__init__wo_kw(TaurusGroupBox,parent)
###        #self.call__init__(TaurusGroupBox,str(self.objectName()))
###        self.setupUi()
###        self.retranslateUi()
###
###
###    def minimumSizeHint(self):
###        return Qt.QSize(260,270)
###
###    def sizeHint(self):
###        return Qt.QSize(260,270)
###
###
###    def setupUi(self):
###
###        self.gridlayout = Qt.QGridLayout(self)
###        self.gridlayout.setObjectName("gridlayout")
###
###        self.m1StateLed_2 = TaurusStateLed(self)
###        self.m1StateLed_2.setLedSize(24)
###        self.m1StateLed_2.setUseParentModel(True)
###        self.m1StateLed_2.setObjectName("m1StateLed_2")
###        self.gridlayout.addWidget(self.m1StateLed_2,0,0,1,1)
###
###        self.taurusValueLabel = TaurusValueLabel(self)
###        self.taurusValueLabel.setFrameShape(Qt.QFrame.NoFrame)
###        self.taurusValueLabel.setFrameShadow(Qt.QFrame.Plain)
###        self.taurusValueLabel.setShowQuality(False)
###        self.taurusValueLabel.setUseParentModel(True)
###        self.taurusValueLabel.setObjectName("taurusValueLabel")
###        self.gridlayout.addWidget(self.taurusValueLabel,0,1,1,1)
###
###        self.TaurusLimitSwitch = TaurusLimitSwitch(self)
###        self.TaurusLimitSwitch.setUseParentModel(True)
###        self.TaurusLimitSwitch.setBoolIndex(2)
###        self.TaurusLimitSwitch.setObjectName("TaurusLimitSwitch")
###        self.gridlayout.addWidget(self.TaurusLimitSwitch,0,3,1,1)
###
###        self.TaurusLimitSwitch_2 = TaurusLimitSwitch(self)
###        self.TaurusLimitSwitch_2.setUseParentModel(True)
###        self.TaurusLimitSwitch_2.setBoolIndex(1)
###        self.TaurusLimitSwitch_2.setObjectName("TaurusLimitSwitch_2")
###        self.gridlayout.addWidget(self.TaurusLimitSwitch_2,0,4,1,1)
###
###        self.m1PositionLabel_2 = TaurusConfigLabel(self)
###        self.m1PositionLabel_2.setUseParentModel(True)
###        self.m1PositionLabel_2.setObjectName("m1PositionLabel_2")
###        self.gridlayout.addWidget(self.m1PositionLabel_2,1,0,1,2)
###
###        self.m1PositionEdit_2 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_2.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_2.setUseParentModel(True)
###        self.m1PositionEdit_2.setObjectName("m1PositionEdit_2")
###        self.gridlayout.addWidget(self.m1PositionEdit_2,1,2,1,1)
###
###        self.m1Position_2 = TaurusValueLabel(self)
###        self.m1Position_2.setUseParentModel(True)
###        self.m1Position_2.setObjectName("m1Position_2")
###        self.gridlayout.addWidget(self.m1Position_2,1,3,1,2)
###
###        self.m1PositionUnitLabel_2 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_2.setUseParentModel(True)
###        self.m1PositionUnitLabel_2.setObjectName("m1PositionUnitLabel_2")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_2,1,5,1,1)
###
###        self.m1VelocityLabel_2 = TaurusConfigLabel(self)
###        self.m1VelocityLabel_2.setUseParentModel(True)
###        self.m1VelocityLabel_2.setObjectName("m1VelocityLabel_2")
###        self.gridlayout.addWidget(self.m1VelocityLabel_2,2,0,1,2)
###
###        self.m1VelocityEdit_2 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1VelocityEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1VelocityEdit_2.setSizePolicy(sizePolicy)
###        self.m1VelocityEdit_2.setUseParentModel(True)
###        self.m1VelocityEdit_2.setObjectName("m1VelocityEdit_2")
###        self.gridlayout.addWidget(self.m1VelocityEdit_2,2,2,1,1)
###
###        self.m1Velocity_2 = TaurusValueLabel(self)
###        self.m1Velocity_2.setUseParentModel(True)
###        self.m1Velocity_2.setObjectName("m1Velocity_2")
###        self.gridlayout.addWidget(self.m1Velocity_2,2,3,1,2)
###
###        self.m1VelocityUnitLabel_2 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1VelocityUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1VelocityUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1VelocityUnitLabel_2.setUseParentModel(True)
###        self.m1VelocityUnitLabel_2.setObjectName("m1VelocityUnitLabel_2")
###        self.gridlayout.addWidget(self.m1VelocityUnitLabel_2,2,5,1,1)
###
###        self.m1PositionLabel_4 = TaurusConfigLabel(self)
###        self.m1PositionLabel_4.setUseParentModel(True)
###        self.m1PositionLabel_4.setObjectName("m1PositionLabel_4")
###        self.gridlayout.addWidget(self.m1PositionLabel_4,3,0,1,2)
###
###        self.m1PositionEdit_4 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_4.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_4.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_4.setUseParentModel(True)
###        self.m1PositionEdit_4.setObjectName("m1PositionEdit_4")
###        self.gridlayout.addWidget(self.m1PositionEdit_4,3,2,1,1)
###
###        self.m1Position_4 = TaurusValueLabel(self)
###        self.m1Position_4.setUseParentModel(True)
###        self.m1Position_4.setObjectName("m1Position_4")
###        self.gridlayout.addWidget(self.m1Position_4,3,3,1,2)
###
###        self.m1PositionUnitLabel_4 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_4.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_4.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_4.setUseParentModel(True)
###        self.m1PositionUnitLabel_4.setObjectName("m1PositionUnitLabel_4")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_4,3,5,1,1)
###
###        self.m1AccelerationLabel_2 = TaurusConfigLabel(self)
###        self.m1AccelerationLabel_2.setUseParentModel(True)
###        self.m1AccelerationLabel_2.setObjectName("m1AccelerationLabel_2")
###        self.gridlayout.addWidget(self.m1AccelerationLabel_2,4,0,1,2)
###
###        self.m1AccelerationEdit_2 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1AccelerationEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1AccelerationEdit_2.setSizePolicy(sizePolicy)
###        self.m1AccelerationEdit_2.setUseParentModel(True)
###        self.m1AccelerationEdit_2.setObjectName("m1AccelerationEdit_2")
###        self.gridlayout.addWidget(self.m1AccelerationEdit_2,4,2,1,1)
###
###        self.m1Acceleration_2 = TaurusValueLabel(self)
###        self.m1Acceleration_2.setUseParentModel(True)
###        self.m1Acceleration_2.setObjectName("m1Acceleration_2")
###        self.gridlayout.addWidget(self.m1Acceleration_2,4,3,1,2)
###
###        self.m1AccelerationUnitLabel_2 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1AccelerationUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1AccelerationUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1AccelerationUnitLabel_2.setUseParentModel(True)
###        self.m1AccelerationUnitLabel_2.setObjectName("m1AccelerationUnitLabel_2")
###        self.gridlayout.addWidget(self.m1AccelerationUnitLabel_2,4,5,1,1)
###
###        self.m1PositionLabel_5 = TaurusConfigLabel(self)
###        self.m1PositionLabel_5.setUseParentModel(True)
###        self.m1PositionLabel_5.setObjectName("m1PositionLabel_5")
###        self.gridlayout.addWidget(self.m1PositionLabel_5,5,0,1,2)
###
###        self.m1PositionEdit_5 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_5.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_5.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_5.setUseParentModel(True)
###        self.m1PositionEdit_5.setObjectName("m1PositionEdit_5")
###        self.gridlayout.addWidget(self.m1PositionEdit_5,5,2,1,1)
###
###        self.m1Position_5 = TaurusValueLabel(self)
###        self.m1Position_5.setUseParentModel(True)
###        self.m1Position_5.setObjectName("m1Position_5")
###        self.gridlayout.addWidget(self.m1Position_5,5,3,1,2)
###
###        self.m1PositionUnitLabel_5 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_5.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_5.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_5.setUseParentModel(True)
###        self.m1PositionUnitLabel_5.setObjectName("m1PositionUnitLabel_5")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_5,5,5,1,1)
###
###        self.m1DecelerationLabel_2 = TaurusConfigLabel(self)
###        self.m1DecelerationLabel_2.setUseParentModel(True)
###        self.m1DecelerationLabel_2.setObjectName("m1DecelerationLabel_2")
###        self.gridlayout.addWidget(self.m1DecelerationLabel_2,6,0,1,2)
###
###        self.m1DecelerationEdit_2 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1DecelerationEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1DecelerationEdit_2.setSizePolicy(sizePolicy)
###        self.m1DecelerationEdit_2.setUseParentModel(True)
###        self.m1DecelerationEdit_2.setObjectName("m1DecelerationEdit_2")
###        self.gridlayout.addWidget(self.m1DecelerationEdit_2,6,2,1,1)
###
###        self.m1Deceleration_2 = TaurusValueLabel(self)
###        self.m1Deceleration_2.setUseParentModel(True)
###        self.m1Deceleration_2.setObjectName("m1Deceleration_2")
###        self.gridlayout.addWidget(self.m1Deceleration_2,6,3,1,2)
###
###        self.m1DecelerationUnitLabel_2 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1DecelerationUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1DecelerationUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1DecelerationUnitLabel_2.setUseParentModel(True)
###        self.m1DecelerationUnitLabel_2.setObjectName("m1DecelerationUnitLabel_2")
###        self.gridlayout.addWidget(self.m1DecelerationUnitLabel_2,6,5,1,1)
###
###        self.m1PositionLabel_6 = TaurusConfigLabel(self)
###        self.m1PositionLabel_6.setUseParentModel(True)
###        self.m1PositionLabel_6.setObjectName("m1PositionLabel_6")
###        self.gridlayout.addWidget(self.m1PositionLabel_6,7,0,1,2)
###
###        self.m1PositionEdit_6 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionEdit_6.sizePolicy().hasHeightForWidth())
###        self.m1PositionEdit_6.setSizePolicy(sizePolicy)
###        self.m1PositionEdit_6.setUseParentModel(True)
###        self.m1PositionEdit_6.setObjectName("m1PositionEdit_6")
###        self.gridlayout.addWidget(self.m1PositionEdit_6,7,2,1,1)
###
###        self.m1Position_6 = TaurusValueLabel(self)
###        self.m1Position_6.setUseParentModel(True)
###        self.m1Position_6.setObjectName("m1Position_6")
###        self.gridlayout.addWidget(self.m1Position_6,7,3,1,2)
###
###        self.m1PositionUnitLabel_6 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1PositionUnitLabel_6.sizePolicy().hasHeightForWidth())
###        self.m1PositionUnitLabel_6.setSizePolicy(sizePolicy)
###        self.m1PositionUnitLabel_6.setUseParentModel(True)
###        self.m1PositionUnitLabel_6.setObjectName("m1PositionUnitLabel_6")
###        self.gridlayout.addWidget(self.m1PositionUnitLabel_6,7,5,1,1)
###
###        self.m1StepPerUnitLabel_2 = TaurusConfigLabel(self)
###        self.m1StepPerUnitLabel_2.setUseParentModel(True)
###        self.m1StepPerUnitLabel_2.setObjectName("m1StepPerUnitLabel_2")
###        self.gridlayout.addWidget(self.m1StepPerUnitLabel_2,8,0,1,2)
###
###        self.m1StepPerUnitEdit_2 = TaurusValueLineEdit(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Fixed,Qt.QSizePolicy.Fixed)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1StepPerUnitEdit_2.sizePolicy().hasHeightForWidth())
###        self.m1StepPerUnitEdit_2.setSizePolicy(sizePolicy)
###        self.m1StepPerUnitEdit_2.setUseParentModel(True)
###        self.m1StepPerUnitEdit_2.setObjectName("m1StepPerUnitEdit_2")
###        self.gridlayout.addWidget(self.m1StepPerUnitEdit_2,8,2,1,1)
###
###        self.m1StepPerUnit_2 = TaurusValueLabel(self)
###        self.m1StepPerUnit_2.setUseParentModel(True)
###        self.m1StepPerUnit_2.setObjectName("m1StepPerUnit_2")
###        self.gridlayout.addWidget(self.m1StepPerUnit_2,8,3,1,2)
###
###        self.m1StepPerUnitUnitLabel_2 = TaurusConfigLabel(self)
###
###        sizePolicy = Qt.QSizePolicy(Qt.QSizePolicy.Expanding,Qt.QSizePolicy.Preferred)
###        sizePolicy.setHorizontalStretch(0)
###        sizePolicy.setVerticalStretch(0)
###        sizePolicy.setHeightForWidth(self.m1StepPerUnitUnitLabel_2.sizePolicy().hasHeightForWidth())
###        self.m1StepPerUnitUnitLabel_2.setSizePolicy(sizePolicy)
###        self.m1StepPerUnitUnitLabel_2.setUseParentModel(True)
###        self.m1StepPerUnitUnitLabel_2.setObjectName("m1StepPerUnitUnitLabel_2")
###        self.gridlayout.addWidget(self.m1StepPerUnitUnitLabel_2,8,5,1,1)
###
###
###    def retranslateUi(self):
###        self.m1StateLed_2.setModel(Qt.QApplication.translate("Dialog", "/State", None, Qt.QApplication.UnicodeUTF8))
###        self.taurusValueLabel.setModel(Qt.QApplication.translate("Dialog", "/State", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch.setLedColor(Qt.QApplication.translate("Dialog", "ORANGE", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch.setModel(Qt.QApplication.translate("Dialog", "/Limit_switches", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch_2.setLedColor(Qt.QApplication.translate("Dialog", "ORANGE", None, Qt.QApplication.UnicodeUTF8))
###        self.TaurusLimitSwitch_2.setModel(Qt.QApplication.translate("Dialog", "/Limit_switches", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_2.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Position?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_2.setModel(Qt.QApplication.translate("Dialog", "/Position", None, Qt.QApplication.UnicodeUTF8))
###        self.m1Position_2.setModel(Qt.QApplication.translate("Dialog", "/Position", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Position?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1VelocityLabel_2.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1VelocityLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Velocity?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1VelocityEdit_2.setModel(Qt.QApplication.translate("Dialog", "/Velocity", None, Qt.QApplication.UnicodeUTF8))
###        self.m1Velocity_2.setModel(Qt.QApplication.translate("Dialog", "/Velocity", None, Qt.QApplication.UnicodeUTF8))
###        self.m1VelocityUnitLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Velocity?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_4.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_4.setModel(Qt.QApplication.translate("Dialog", "/Acceleration?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_4.setModel(Qt.QApplication.translate("Dialog", "/Acceleration", None, Qt.QApplication.UnicodeUTF8))
###        self.m1Position_4.setModel(Qt.QApplication.translate("Dialog", "/Acceleration", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_4.setModel(Qt.QApplication.translate("Dialog", "/Acceleration?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1AccelerationLabel_2.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1AccelerationLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Deceleration?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1AccelerationEdit_2.setModel(Qt.QApplication.translate("Dialog", "/Deceleration", None, Qt.QApplication.UnicodeUTF8))
###        self.m1Acceleration_2.setModel(Qt.QApplication.translate("Dialog", "/Deceleration", None, Qt.QApplication.UnicodeUTF8))
###        self.m1AccelerationUnitLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Deceleration?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_5.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_5.setModel(Qt.QApplication.translate("Dialog", "/Offset?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_5.setModel(Qt.QApplication.translate("Dialog", "/Offset", None, Qt.QApplication.UnicodeUTF8))
###        self.m1Position_5.setModel(Qt.QApplication.translate("Dialog", "/Offset", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_5.setModel(Qt.QApplication.translate("Dialog", "/Offset?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1DecelerationLabel_2.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1DecelerationLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Base_rate?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1DecelerationEdit_2.setModel(Qt.QApplication.translate("Dialog", "/Base_rate", None, Qt.QApplication.UnicodeUTF8))
###        self.m1Deceleration_2.setModel(Qt.QApplication.translate("Dialog", "/Base_rate", None, Qt.QApplication.UnicodeUTF8))
###        self.m1DecelerationUnitLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Base_rate?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_6.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionLabel_6.setModel(Qt.QApplication.translate("Dialog", "/Step_per_unit?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionEdit_6.setModel(Qt.QApplication.translate("Dialog", "/Step_per_unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1Position_6.setModel(Qt.QApplication.translate("Dialog", "/Step_per_unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1PositionUnitLabel_6.setModel(Qt.QApplication.translate("Dialog", "/Step_per_unit?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitLabel_2.setSuffixText(Qt.QApplication.translate("Dialog", ":", None, Qt.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Backlash?configuration=label", None, Qt.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitEdit_2.setModel(Qt.QApplication.translate("Dialog", "/Backlash", None, Qt.QApplication.UnicodeUTF8))
###        self.m1StepPerUnit_2.setModel(Qt.QApplication.translate("Dialog", "/Backlash", None, Qt.QApplication.UnicodeUTF8))
###        self.m1StepPerUnitUnitLabel_2.setModel(Qt.QApplication.translate("Dialog", "/Backlash?configuration=unit", None, Qt.QApplication.UnicodeUTF8))
###
###    ##############################################################################
###    ## VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
###    ##
###    ## I CAN NOT INHERIT FROM TAUGROUPBOX !
###    ## SO ALL THE STUFF BELOW IS NECESSARY
###
###
###
###    def defineStyle(self):
###        palette = Qt.QPalette()
###        self.setPalette(palette)
###        self.updateStyle()
###
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # TaurusBaseWidget over writing
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###
###    def getDisplayValue(self):
###        return (self._prefix or '') + TaurusBaseWidget.getDisplayValue(self) + (self._suffix or '')
###
###    def isReadOnly(self):
###        return True
###
###    def updateStyle(self):
###        if self.getShowQuality():
###            self.setAutoFillBackground(True)
###            #TODO: get quality/state from model and update accordingly
###        else:
###            self.setAutoFillBackground(False)
###            #TODO: restore colors
###        self.update()
###
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###    # QT properties
###    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
###
###    def getPrefixText(self):
###        return self._prefix
###
###    @Qt.pyqtSignature("setPrefixText(QString)")
###    def setPrefixText(self,prefix):
###        self._prefix = prefix
###        self.fireValueChanged()
###
###    def getSuffixText(self):
###        return self._suffix
###
###    @Qt.pyqtSignature("setSuffixText(QString)")
###    def setSuffixText(self,suffix):
###        self._suffix = suffix
###        self.fireValueChanged()
###
###    model = Qt.pyqtProperty("QString", TaurusBaseWidget.getModel,
###                                TaurusBaseWidget.setModel, TaurusBaseWidget.resetModel)
###    useParentModel = Qt.pyqtProperty("bool", TaurusBaseWidget.getUseParentModel,
###                                         TaurusBaseWidget.setUseParentModel,
###                                         TaurusBaseWidget.resetUseParentModel)
###    showQuality = Qt.pyqtProperty("bool", TaurusBaseWidget.getShowQuality,
###                                      TaurusBaseWidget.setShowQuality,
###                                      TaurusBaseWidget.resetShowQuality)
###    showText = Qt.pyqtProperty("bool", TaurusBaseWidget.getShowText,
###                                   TaurusBaseWidget.setShowText,
###                                   TaurusBaseWidget.resetShowText)
###    prefixText = Qt.pyqtProperty("QString", getPrefixText, setPrefixText,
###                                     doc="prefix text (optional)")
###    suffixText = Qt.pyqtProperty("QString", getSuffixText, setSuffixText,
###                                    doc="suffix text (optional)")
###
###
###

if __name__ == "__main__":

    import sys
    app = Qt.QApplication(sys.argv)

    form = TaurusMotorH()
    form.setModel(sys.argv[1])

    form.show()
    sys.exit(app.exec_())



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Pool library. It defines the base classes
for"""

__all__ = ["PoolMeasurementGroup"]

__docformat__ = 'restructuredtext'

from taurus.core.taurusvalidator import AttributeNameValidator

from sardana import State, ElementType, \
    TYPE_EXP_CHANNEL_ELEMENTS, TYPE_TIMERABLE_ELEMENTS
from sardana.sardanaevent import EventType
from sardana.pool.pooldefs import AcqMode, AcqTriggerType
from sardana.pool.poolgroupelement import PoolGroupElement
from sardana.pool.poolacquisition import PoolAcquisition
from sardana.pool.poolexternal import PoolExternalObject

from sardana.taurus.core.tango.sardana import PlotType, Normalization

#----------------------------------------------
# Measurement Group Configuration information
#----------------------------------------------
# dict <str, obj> with (at least) keys:
#    - 'timer' : the timer channel name / timer channel id
#    - 'monitor' : the monitor channel name / monitor channel id
#    - 'controllers' : dict<Controller, dict> where:
#        - key: ctrl
#        - value: dict<str, dict> with (at least) keys:
#            - 'units': dict<str, dict> with (at least) keys:
#                - 'id' : the unit ID inside the controller
#                - 'timer' : the timer channel name / timer channel id
#                - 'monitor' : the monitor channel name / monitor channel id
#                - 'trigger_type' : 'Gate'/'Software'
#                - 'channels' where value is a dict<str, obj> with (at least) keys:
#                    - 'id' : the channel name ( channel id )
#                    optional keys:
#                    - 'enabled' : True/False (default is True)
#                    any hints:
#                    - 'output' : True/False (default is True)
#                    - 'plot_type' : 'No'/'1D'/'2D' (default is 'No')
#                    - 'plot_axes' : list<str> 'where str is channel name/'step#/'index#' (default is [])
#                    - 'label' : prefered label (default is channel name)
#                    - 'scale' : <float, float> with min/max (defaults to channel
#                                range if it is defined
#                    - 'plot_color' : int representing RGB
#    optional keys:
#    - 'label' : measurement group label (defaults to measurement group name)
#    - 'description' : measurement group description

# <MeasurementGroupConfiguration>
#  <timer>UxTimer</timer>
#  <monitor>CT1</monitor>
# </MeasurementGroupConfiguration>

# Example: 2 NI cards, where channel 1 of card 1 is wired to channel 1 of card 2
# at configuration time we should set:
# ctrl.setPar( <unit>, <parameter name>, <parameter value> )
# ni0ctrl.setCtrlPar(0, 'trigger_type', AcqTriggerType.Software)
# ni0ctrl.setCtrlPar(0, 'timer', 1) # channel 1 is the timer
# ni0ctrl.setCtrlPar(0, 'monitor', 4) # channel 4 is the monitor
# ni1ctrl.setCtrlPar(0, 'trigger_type', AcqTriggerType.ExternalTrigger)
# ni1ctrl.setCtrlPar(0, 'master', 0)

# when we count for 1.5 seconds:
# ni1ctrl.Load(1.5)
# ni0ctrl.Load(1.5)
# ni1ctrl.Start()
# ni0ctrl.Start()

"""

"""

class PoolMeasurementGroup(PoolGroupElement):

    DFT_DESC = 'General purpose measurement group'

    def __init__(self, **kwargs):
        self._integration_time = None
        self._monitor_count = None
        self._acquisition_mode = AcqMode.Timer
        self._config = None
        self._config_dirty = True
        kwargs['elem_type'] = ElementType.MeasurementGroup
        PoolGroupElement.__init__(self, **kwargs)
        self.set_configuration(kwargs.get('configuration'))

    def _create_action_cache(self):
        acq_name = "%s.Acquisition" % self._name
        return PoolAcquisition(self, acq_name)

    def _calculate_element_state(self, elem, elem_state_info):
        if elem.get_type() == ElementType.ZeroDExpChannel:
            if elem_state_info[0] == State.Moving:
                elem_state_info = State.On, elem_state_info[1]
        return PoolGroupElement._calculate_element_state(self, elem,
                                                         elem_state_info)

    def on_element_changed(self, evt_src, evt_type, evt_value):
        name = evt_type.name
        if name == 'state':
            if evt_src.get_type() == ElementType.ZeroDExpChannel:
                # 0D channels are "passive", which means they cannot contribute
                # to set the measurement group into a moving state
                if evt_value in (State.On, State.Moving):
                    return
            state, status = self._calculate_states()
            self.set_state(state, propagate=2)
            self.set_status("\n".join(status))

    def get_pool_controllers(self):
        return self.get_acquisition().get_pool_controllers()

    def get_pool_controller_by_name(self, name):
        name = name.lower()
        for ctrl in self.get_pool_controllers():
            if ctrl.name.lower() == name or ctrl.full_name.lower() == name:
                return ctrl

    # --------------------------------------------------------------------------
    # configuration
    # --------------------------------------------------------------------------

    def _is_managed_element(self, element):
        return element.get_type() in TYPE_EXP_CHANNEL_ELEMENTS

    def _build_channel_defaults(self, channel_data, channel):
        """Fills the channel default values for the given channel dictionary"""

        external_from_name = isinstance(channel, (str, unicode))
        ndim = None
        instrument = None
        if external_from_name:
            name = full_name = source = channel
        else:
            name = channel.name
            full_name = channel.full_name
            source = channel.get_source()
            ndim = None
            instrument = None
            ctype = channel.get_type()
            if ctype != ElementType.External:
                instrument = channel.instrument
            if ctype == ElementType.CTExpChannel:
                ndim = 0
            elif ctype == ElementType.PseudoCounter:
                ndim = 0
            elif ctype == ElementType.ZeroDExpChannel:
                ndim = 0
            elif ctype == ElementType.OneDExpChannel:
                ndim = 1
            elif ctype == ElementType.TwoDExpChannel:
                ndim = 2
            elif ctype == ElementType.External:
                config = channel.get_config()
                if config is not None:
                    ndim = int(config.data_format)
            elif ctype == ElementType.IORegister:
                ndim = 0

        # Definitively should be initialized by measurement group
        # index MUST be here already (asserting this in the following line)
        channel_data['index'] = channel_data['index']
        channel_data['name'] = channel_data.get('name', name)
        channel_data['full_name'] = channel_data.get('full_name', full_name)
        channel_data['source'] = channel_data.get('source', source)
        channel_data['enabled'] = channel_data.get('enabled', True)
        channel_data['label'] = channel_data.get('label', channel_data['name'])
        channel_data['instrument'] = channel_data.get('instrument', getattr(instrument, 'name', None))
        channel_data['ndim'] = ndim
        # Probably should be initialized by measurement group
        channel_data['output'] = channel_data.get('output', True)

        # Perhaps should NOT be initialized by measurement group
        channel_data['plot_type'] = channel_data.get('plot_type', PlotType.No)
        channel_data['plot_axes'] = channel_data.get('plot_axes', [])
        channel_data['conditioning'] = channel_data.get('conditioning', '')
        channel_data['normalization'] = channel_data.get('normalization', Normalization.No)

        return channel_data

    def _build_configuration(self):
        """Builds a configuration object from the list of elements"""
        config = {}
        user_elements = self.get_user_elements()
        ctrls = self.get_pool_controllers()

        # find the first CT
        first_timerable = None
        for elem in user_elements:
            if elem.get_type() in TYPE_TIMERABLE_ELEMENTS:
                first_timerable = elem
                break
        if first_timerable is None:
            raise Exception("It is not possible to construct a measurement "
                            "group without at least one timer able channel "
                            "(Counter/timer, 1D or 2D)")
        g_timer = g_monitor = first_timerable
        config['timer'] = g_timer
        config['monitor'] = g_monitor
        config['controllers'] = controllers = {}

        external_user_elements = []
        for index, element in enumerate(user_elements):
            elem_type = element.get_type()
            if elem_type == ElementType.External:
                external_user_elements.append((index, element))
                continue

            ctrl = element.controller
            ctrl_data = controllers.get(ctrl)

            # attention: following lines are only prepared for 1 unit per
            # controller
            if ctrl_data is None:
                controllers[ctrl] = ctrl_data = {}
                ctrl_data['units'] = units = {}
                units['0'] = unit_data = {}
                unit_data['id'] = 0
                unit_data['channels'] = channels = {}
                if elem_type in TYPE_TIMERABLE_ELEMENTS:
                    elements = ctrls[ctrl]
                    if g_timer in elements:
                        unit_data['timer'] = g_timer
                    else:
                        unit_data['timer'] = elements[0]
                    if g_monitor in elements:
                        unit_data['monitor'] = g_monitor
                    else:
                        unit_data['monitor'] = elements[0]
                    unit_data['trigger_type'] = AcqTriggerType.Software
            else:
                channels = ctrl_data['units']['0']['channels']
            channels[element] = channel_data = {}
            channel_data['index'] = user_elements.index(element)
            channel_data = self._build_channel_defaults(channel_data, element)
        config['label'] = self.name
        config['description'] = self.DFT_DESC

        if len(external_user_elements) > 0:
            controllers['__tango__'] = ctrl_data = {}
            ctrl_data['units'] = units = {}
            units['0'] = unit_data = {}
            unit_data['id'] = 0
            unit_data['channels'] = channels = {}
            for index, element in external_user_elements:
                channels[element] = channel_data = {}
                channel_data['index'] = index
                channel_data = self._build_channel_defaults(channel_data, element)
        return config

    def set_configuration(self, config=None, propagate=1):
        if config is None:
            config = self._build_configuration()
        else:
            # create a configuration based on a new configuration
            user_elem_ids = {}
            pool = self.pool
            for c, c_data in config['controllers'].items():
                external = isinstance(c, (str, unicode))
                # attention: following line only prepared for 1 unit per controller
                for channel_data in c_data['units']['0']['channels'].values():
                    if external:
                        element = id = channel_data['full_name']
                        channel_data['source'] = id
                    else:
                        element = pool.get_element_by_full_name(channel_data['full_name'])
                        id = element.id
                    user_elem_ids[channel_data['index']] = id
                    channel_data = self._build_channel_defaults(channel_data, element)
            indexes = sorted(user_elem_ids.keys())
            assert indexes == range(len(indexes))
            self.set_user_element_ids([ user_elem_ids[idx] for idx in indexes ])

        # checks
        g_timer, g_monitor = config['timer'], config['monitor']

        # attention: following line only prepared for 1 unit per controller
        timer_ctrl_data = config['controllers'][g_timer.controller]['units']['0']
        if timer_ctrl_data['timer'] != g_timer:
            self.warning('unit timer and global timer mismatch. '
                         'Using global timer')
            self.debug('For controller %s, timer is defined as channel %s. '
                       'The global timer is set to channel %s which belongs '
                       'to the same controller', g_timer.controller.name,
                       timer_ctrl_data['timer'].name, g_timer.name)
            timer_ctrl_data['timer'] = g_timer

        # attention: following line only prepared for 1 unit per controller
        monitor_ctrl_data = config['controllers'][g_monitor.controller]['units']['0']
        if monitor_ctrl_data['monitor'] != g_monitor:
            self.warning('unit monitor and global monitor mismatch. '
                         'Using global monitor')
            self.debug('For controller %s, monitor is defined as channel %s. '
                       'The global timer is set to channel %s which belongs '
                       'to the same controller', g_monitor.controller.name,
                       monitor_ctrl_data['monitor'].name, g_monitor.name)
            monitor_ctrl_data['monitor'] != g_monitor

        self._config = config
        self._config_dirty = True
        if not propagate:
            return
        self.fire_event(EventType("configuration", priority=propagate), config)

    def set_configuration_from_user(self, cfg, propagate=1):
        config = {}
        user_elements = self.get_user_elements()
        pool = self.pool

        timer_name = cfg.get('timer', user_elements[0].full_name)
        monitor_name = cfg.get('monitor', user_elements[0].full_name)
        config['timer'] = pool.get_element_by_full_name(timer_name)
        config['monitor'] = pool.get_element_by_full_name(monitor_name)
        config['controllers'] = controllers = {}

        for c_name, c_data in cfg['controllers'].items():
            # discard controllers which don't have items (garbage)
            ch_count = 0
            for u_data in c_data['units'].values():
                ch_count += len(u_data['channels'])
            if ch_count == 0:
                continue

            external = c_name.startswith('__')
            if external:
                ctrl = c_name
            else:
                ctrl = pool.get_element_by_full_name(c_name)
                assert ctrl.get_type() == ElementType.Controller
            controllers[ctrl] = ctrl_data = {}
            ctrl_data['units'] = units = {}
            for u_id, u_data in c_data['units'].items():
                # discard units which don't have items (garbage)
                if len(u_data['channels']) == 0:
                    continue
                units[u_id] = unit_data = dict(u_data)
                unit_data['id'] = u_data.get('id', u_id)
                if not external and ctrl.is_timerable():
                    unit_data['timer'] = pool.get_element_by_full_name(u_data['timer'])
                    unit_data['monitor'] = pool.get_element_by_full_name(u_data['monitor'])
                    unit_data['trigger_type'] = u_data['trigger_type']
                unit_data['channels'] = channels = {}
                for ch_name, ch_data in u_data['channels'].items():
                    if external:
                        validator = AttributeNameValidator()
                        params = validator.getParams(ch_data['full_name'])
                        params['pool'] = self.pool
                        channel = PoolExternalObject(**params)
                    else:
                        channel = pool.get_element_by_full_name(ch_name)
                    channels[channel] = dict(ch_data)

        config['label'] = cfg.get('label', self.name)
        config['description'] = cfg.get('description', self.DFT_DESC)
        self.set_configuration(config, propagate=propagate)

    def get_configuration(self):
        return self._config

    def get_user_configuration(self):
        cfg = self.get_configuration()
        config = {}

        config['timer'] = cfg['timer'].full_name
        config['monitor'] = cfg['monitor'].full_name
        config['controllers'] = controllers = {}

        for c, c_data in cfg['controllers'].items():
            ctrl_name = c
            if not isinstance(c, (str, unicode)):
                ctrl_name = c.full_name
            external = ctrl_name.startswith('__')
            controllers[ctrl_name] = ctrl_data = {}
            ctrl_data['units'] = units = {}
            for u_id, u_data in c_data['units'].items():
                units[u_id] = unit_data = {}
                unit_data['id'] = u_data['id']
                if not external and c.is_timerable():
                    if u_data.has_key('timer'):
                        unit_data['timer'] = u_data['timer'].full_name
                    if u_data.has_key('monitor'):
                        unit_data['monitor'] = u_data['monitor'].full_name
                    if u_data.has_key('trigger_type'):
                        unit_data['trigger_type'] = u_data['trigger_type']
                unit_data['channels'] = channels = {}
                for ch, ch_data in u_data['channels'].items():
                    channels[ch.full_name] = dict(ch_data)

        config['label'] = cfg['label']
        config['description'] = cfg['description']
        return config

    def load_configuration(self, force=False):
        """Loads the current configuration to all involved controllers"""
        cfg = self.get_configuration()
        g_timer, g_monitor = cfg['timer'], cfg['monitor']

        for ctrl, ctrl_data in cfg['controllers'].items():
            # skip external channels
            if type(ctrl) is str:
                continue
            if ctrl.operator == self and not force and not self._config_dirty:
                continue
            ctrl.operator = self
            if ctrl.is_timerable():
                for unit, unit_data in ctrl_data['units'].items():
                    #if ctrl == g_timer.controller:
                    #    ctrl.set_ctrl_par('timer', g_timer.axis)
                    #if ctrl == g_monitor.controller:
                    #    ctrl.set_ctrl_par('monitor', g_monitor.axis)
                    ctrl.set_ctrl_par('timer', unit_data['timer'].axis)
                    ctrl.set_ctrl_par('monitor', unit_data['monitor'].axis)
                    ctrl.set_ctrl_par('trigger_type', unit_data['trigger_type'])

        self._config_dirty = False

    def get_timer(self):
        return self.get_configuration()['timer']

    timer = property(get_timer)

    # --------------------------------------------------------------------------
    # integration time
    # --------------------------------------------------------------------------

    def get_integration_time(self):
        return self._integration_time

    def set_integration_time(self, integration_time, propagate=1):
        self._integration_time = integration_time
        if not propagate:
            return
        self.fire_event(EventType("integration_time", priority=propagate),
                        integration_time)

    integration_time = property(get_integration_time, set_integration_time,
                                doc="the current integration time")

    # --------------------------------------------------------------------------
    # integration time
    # --------------------------------------------------------------------------

    def get_monitor_count(self):
        return self._monitor_count

    def set_monitor_count(self, monitor_count, propagate=1):
        self._monitor_count = monitor_count
        if not propagate:
            return
        self.fire_event(EventType("monitor_count", priority=propagate),
                        monitor_count)

    monitor_count = property(get_monitor_count, set_monitor_count,
                             doc="the current monitor count")

    # --------------------------------------------------------------------------
    # acquisition mode
    # --------------------------------------------------------------------------

    def get_acquisition_mode(self):
        return self._acquisition_mode

    def set_acquisition_mode(self, acquisition_mode, propagate=1):
        self._acquisition_mode = acquisition_mode
        if not propagate:
            return
        self.fire_event(EventType("acquisition_mode", priority=propagate),
                        acquisition_mode)

    acquisition_mode = property(get_acquisition_mode, set_acquisition_mode,
                                doc="the current acquisition mode")

    # --------------------------------------------------------------------------
    # acquisition
    # --------------------------------------------------------------------------

    def start_acquisition(self, value=None, multiple=1):
        self._aborted = False
        if not self._simulation_mode:
            # load configuration into controller(s) if necessary
            self.load_configuration()
            # start acquisition
            kwargs = dict(head=self, config=self._config, multiple=multiple)
            if self.acquisition_mode == AcqMode.Timer:
                kwargs["integ_time"] = self._integration_time
            elif self.acquisition_mode == AcqMode.Monitor:
                kwargs["monitor_count"] = self._monitor_count
            self.acquisition.run(**kwargs)

    def set_acquisition(self, acq_cache):
        self.set_action_cache(acq_cache)

    def get_acquisition(self):
        return self.get_action_cache()

    acquisition = property(get_acquisition, doc="acquisition object")




#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""
sequenceeditor.py: 
"""

import sys
from copy import deepcopy

import PyTango

from taurus.external.qt import Qt
from taurus import Device
from taurus.qt.qtgui.container import TaurusWidget, TaurusMainWindow, TaurusBaseContainer
from taurus.qt.qtgui.display import TaurusLed
from taurus.qt.qtgui.dialog import TaurusMessageBox
from taurus.qt.qtgui.resource import getIcon, getThemeIcon

from sardana.taurus.core.tango.sardana import macro
from sardana.taurus.core.tango.sardana.macro import MacroRunException
from sardana.taurus.qt.qtgui.extra_macroexecutor.macroparameterseditor import ParamEditorManager, ParamEditorModel, StandardMacroParametersEditor

from .favouriteseditor import FavouritesMacrosEditor, HistoryMacrosViewer
from .common import MacroComboBox, MacroExecutionWindow, standardPlotablesFilter


class MacroProgressBar(Qt.QProgressBar):

    def __init__(self, parent=None):
        Qt.QProgressBar.__init__(self, parent)

class SpockCommandWidget(Qt.QLineEdit, TaurusBaseContainer):
    def __init__(self, name, parent=None, designMode=False):
        #self.newValue - is used as a flag to indicate whether a controlUp controlDown actions are used to iterate existing element or put new one
        #self.disableEditMode - flag, used to disable edition, when user enters name of the macro which is not valid (not allowed to edit in the yellow line)
        #                   switches off validation
        #disableSpockCommandUpdate - flag, it disables updates of the model when macro is edited by macroEditor

        Qt.QLineEdit.__init__(self, parent)
        TaurusBaseContainer.__init__(self, name, parent, designMode)

        self._model = None
        self.setFont(Qt.QFont("Courier", 9))
        palette = Qt.QPalette()
        palette.setColor(Qt.QPalette.Base, Qt.QColor('yellow'))
        self.setPalette(palette)
        self.currentIndex = Qt.QModelIndex()
        self.newValue = False
        self.disableSpockCommandUpdate = False
        self.disableEditMode = True
        self.setEnabled(False)

        self.setActions()
        self.connect(self, Qt.SIGNAL("textChanged(const QString &)"), self.textChanged)
        self.connect(self, Qt.SIGNAL("returnPressed()"), self.returnPressed)

    def setActions(self):
        self._downAction = Qt.QAction("downAction", self)
        self._upAction = Qt.QAction("upAction", self)
        self._ctrlDownAction = Qt.QAction("controlDownAction", self)
        self._ctrlUpAction = Qt.QAction("controlUpAction", self)

        self._ctrlDownAction.setShortcut(Qt.QKeySequence(Qt.Qt.CTRL + Qt.Qt.Key_Down))
        self._ctrlUpAction.setShortcut(Qt.QKeySequence(Qt.Qt.CTRL + Qt.Qt.Key_Up))
        self._downAction.setShortcuts([Qt.Qt.Key_Down])
        self._upAction.setShortcuts([Qt.Qt.Key_Up])

        self._ctrlDownAction.setShortcutContext (Qt.Qt.WidgetShortcut)
        self._ctrlUpAction.setShortcutContext (Qt.Qt.WidgetShortcut)
        self._downAction.setShortcutContext (Qt.Qt.WidgetShortcut)
        self._upAction.setShortcutContext (Qt.Qt.WidgetShortcut)

        self.addAction(self._ctrlDownAction)
        self.addAction(self._ctrlUpAction)
        self.addAction(self._downAction)
        self.addAction(self._upAction)
        self.connect(self._downAction, Qt.SIGNAL("triggered()"), self.downAction)
        self.connect(self._upAction, Qt.SIGNAL("triggered()"), self.upAction)
        self.connect(self._ctrlDownAction, Qt.SIGNAL("triggered()"), self.controlDownAction)
        self.connect(self._ctrlUpAction, Qt.SIGNAL("triggered()"), self.controlUpAction)

    def setCommand(self):
        command = self._model.toSpockCommand()
        command = command.replace("None", "").strip()
        if not self.disableSpockCommandUpdate:
            self.setText(command)

    def setModel(self, model):
        enable = bool(model)
        self.disableEditMode = not enable
        self.setEnabled(enable)
        self._model = model
        self.connect(self._model, Qt.SIGNAL("dataChanged(QModelIndex,QModelIndex)"), self.setCommand)
        self.connect(self._model, Qt.SIGNAL("modelReset()"), self.setCommand)

    def model(self):
        return self._model

    def getIndex(self, elementNumber=-1):
        #Returns QModelIndex of the required element (number of single parameter). If the elementNumber == -1 next single parameter index is returned.
        if elementNumber == -1:
            ix = self.currentIndex
            elementNumber = 1
        elif elementNumber == 0:
            return Qt.QModelIndex()
        else:
            ix = Qt.QModelIndex()

        (col, row, parentIdx) = (ix.column(), ix.row(), ix.parent())
        #to start from second column
        if col == -1 and row == -1:
            ix = self.forwardIdx(0, 1, ix)
        for i in range (0, elementNumber):
            #This condition in case we start tabbing with cursor on first column
            if col == 0:
                currentNode = self.model().nodeFromIndex(ix)
                if isinstance(currentNode, macro.SingleParamNode):
                    nextIdx = self.forwardIdx(row, 1, parentIdx)
                else:
                    nextIdx = self.forwardIdx(0, 1, ix)
            else:
                nextIdx = self.forwardIdx(row + 1, 1, parentIdx)
            #this condition in case there is no next index and we want to pass focus
            #to next widget in parent obj
            if nextIdx == "term":
                return Qt.QModelIndex()
            ix = nextIdx
            (col, row, parentIdx) = (ix.column(), ix.row(), ix.parent())
        return ix

    def forwardIdx(self, row, col, parentIdx):
        #This method is moving down the tree to get next SingleParamNode index.
        try:
            proposalIdx = self.model().index(row, col, parentIdx)
        except AssertionError:

            if parentIdx.row() == -1:
                return Qt.QModelIndex()
            grandParentIdx = parentIdx.parent()
            return self.forwardIdx(parentIdx.row() + 1, col, grandParentIdx)

        proposalNode = self.model().nodeFromIndex(proposalIdx)

        if isinstance(proposalNode, macro.SingleParamNode):
            return proposalIdx
        elif isinstance(proposalNode, macro.RepeatNode):
            return self.forwardIdx(0, 1, proposalIdx)
        elif isinstance(proposalNode, macro.RepeatParamNode):
            if len(proposalNode) > 0:
                return self.forwardIdx(0, 1, proposalIdx)
            else:
                return self.forwardIdx(row + 1, col, proposalIdx)

        elif not proposalIdx.isValid():
            proposalIdx = parentIdx.sibling(parentIdx.row() + 1, 0)

            if proposalIdx.isValid():
                proposalIdx = proposalIdx.child(0, 1)
            else:
                while not proposalIdx.isValid():
                    parentIdx = parentIdx.parent()
                    if not parentIdx.isValid():

                        return Qt.QModelIndex()
                    proposalIdx = parentIdx.sibling(parentIdx.row() + 1, 1)

            return proposalIdx

    def validateAllExpresion(self, secValidation=False):
        #This method is responsible for full validation of the macro. It is executed whenever the text is changed (when user edits values).
        #Validation starts with checking if the macro (name) is valid.
        #Next steps:
        #1. Validates every SingleParamNode and counts how many there are in the macro.
        #2. If there are more SingleParamNodes than entered values it will check if there is RepeatParamNode.
        #   If there is RepeatParamNode it will check if its RepeatNodes can be deleted.
        #3. If there are more values entered than SingleParamNodes in macro it will check if there is RepeatParamNode.
        #   If there is it will try to add new RepeatNode.

        if self.model() is None:
            raise RuntimeError('Door must be set in order to use the macroexecutor.')

        self.currentIndex = Qt.QModelIndex()
        mlist = str(self.text()).split()
        problems = []
        try:
            if str(mlist[0]) != str(self.model().root().name()):
                try:
                    self.getModelObj().validateMacroName(str(mlist[0]))
                    self.validateMacro(mlist[0])
                    self.updateMacroEditor(mlist[0])
                    if not secValidation:
                        self.validateAllExpresion(True)

                except Exception as e:
                    if self.disableEditMode:
                        self.updateMacroEditor(mlist[0])
                        raise Exception(e)
                    message = e[0]
                    #raise Exception(e)
                    problems.append(message)

        except IndexError:
                problems.append("<b>Macro<\b> is missing!")
                self.setStyleSheet("")
                self.setToolTip('<br>'.join(problems))
                return

        self.currentIndex = Qt.QModelIndex()
        ix = self.getIndex()
        self.currentIndex = ix
        counter = 1
        while not ix == Qt.QModelIndex():
            try:
                propValue = mlist[counter]
                try:
                    self.validateOneValue(propValue)
                    self.model().setData(self.currentIndex, Qt.QVariant(propValue))
                except Exception as e:
                    self.model().setData(self.currentIndex, Qt.QVariant('None'))
                    txt = str(Qt.from_qvariant(ix.sibling(ix.row(), 0).data(), str))
                    message = "<b>" + txt + "</b> " + e[0]
                    problems.append(message)
            except IndexError:
                txt = str(Qt.from_qvariant(ix.sibling(ix.row(), 0).data(), str))
                problems.append("<b>" + txt + "</b> is missing!")

                data = str(Qt.from_qvariant(ix.data(), str))
                if data != 'None':
                    self.model().setData(self.currentIndex, Qt.QVariant('None'))
            counter += 1
            ix = self.getIndex()
            self.currentIndex = ix

        if len(mlist) > counter:  #if there are more values than parameters
            repeatNode = None
            for i in self.model().root().params():
                repeatNode = i
                if isinstance(repeatNode, macro.RepeatParamNode):
                    index = self.findParamRepeat(i)
                    self.currentIndex = self.model()._insertRow(index)
                    nn = self.model().nodeFromIndex(self.currentIndex)
                    self.emit(Qt.SIGNAL("expandTree"))
                    ix = self.getIndex()
                    if not secValidation:
                        self.validateAllExpresion(True)
                        return

                repeatNode = None
            if repeatNode == None:
                problems.append("Too many values.")

        elif counter - len(mlist) >= 1:
            repeatNode = None
            node = None
            for i in self.model().root().params():
                repeatNode = i
                if isinstance(repeatNode, macro.RepeatParamNode):
                    index = self.findParamRepeat(i)
                    node = self.model().nodeFromIndex(index)
                    sub = len(node.child(0))
                    break
                repeatNode = None

            if repeatNode is not None:
                while counter - len(mlist) > sub - 1:
                    if len(node.children()) == 1 and node.isReachedMin():
                        break
                    self.model()._removeRow(index.child(len(node.children()) - 1, 0))
                    counter -= sub

                if not secValidation:
                    self.validateAllExpresion(True)
                    return

        if len(problems) == 0:
            self.setStyleSheet('SpockCommandWidget {background-color: %s; color: %s; border: %s; border-radius: %s}' % ('yellow', 'black', '3px solid green', '5px'))
            self.setToolTip("")
        else:
            self.setStyleSheet("")
            self.setToolTip('<br>'.join(problems))
        return

    def findParamRepeat(self, repeatNode):
        #Method which finds index of given ParamRepeatNode in the macro.
        children = self.model().root().children()
        occ = children.count(repeatNode)
        idx = 0
        for i in range(0, occ):
            idx = children.index(repeatNode, idx)
        index = self.model().index(idx , 0, Qt.QModelIndex())
        return index

    def validateOneValue(self, value):
        #Validates value of a SingleParamNode of a currentIndex
        paramNode = deepcopy(self.model().nodeFromIndex(self.currentIndex))
        paramNode.setValue(value)
        return self.getModelObj().validateSingleParam(paramNode)

    def returnPressed(self):
        #SLOT called when return is pressed
        if self.toolTip() == "":
            self.emit(Qt.SIGNAL("pressedReturn"))
        else:
            raise Exception("Cannot start macro. Please correct following mistakes: <br>" + self.toolTip())

    def textChanged(self, strs):
        #SLOT called when QLineEdit text is changed
        if strs == "":
            self.updateMacroEditor("")

        if not self.disableEditMode and self.disableSpockCommandUpdate:
            self.validateAllExpresion()
        else:
            txt_parts = str(self.text()).split()
            if len(txt_parts) == 0:
                return
            try:
                if self.validateMacro(txt_parts[0]):
                    self.validateAllExpresion()
            except:
                self.setToolTip("Read Mode")

    def validateMacro(self, value):
        #Method which ivestigates if the macro can be edited using yellow line.
        #It cannot be executed when: 1. there are more than 1 ParamRepeatNodes,
        #2. There is a ParamRepeatNode inside ParamRepeatNodem
        #3. After ParamRepeatNode there are other nodes

        macroNode = self.getModelObj().getMacroNodeObj(str(value))
        if macroNode is None: return False
        t = [child for child in macroNode.children() if isinstance(child, macro.RepeatParamNode)]
        if len(t) > 1:
            self.disableEditMode = True
            raise Exception('Macro <b> %s </b> cannot be edited using yellow line.<br>It contains more than 1 paramRepeat node. <br>Please use Macro Editor Widget to edit and execute this macro.' % str(value))
        elif len(t) == 1:
            if len([child for child in t[0].children() if isinstance(child, macro.RepeatParamNode)]) > 0:
                self.disableEditMode = True
                raise Exception('Macro <b> %s </b> cannot be edited using yellow line.<br>It contains paramRepeat node inside paramRepeat node. <br>Please use Macro Editor Widget to edit and execute this macro.' % str(value))
            else:
                if macroNode.children().index(t[0]) != len(macroNode.children()) - 1 :
                    self.disableEditMode = True
                    raise Exception('Macro <b> %s </b> cannot be edited using yellow line.<br>It contains paramRepeat node but not as a last parameter. <br>Please use Macro Editor Widget to edit and execute this macro.' % str(value))
        self.disableEditMode = False
        return True

    def downAction(self):
        #Goes down in the history list of executed macros.
        #self.disableSpockCommandUpdate flag is used to allow updating yellow line when model is changed. (when new row in history is chosen)

        self.disableSpockCommandUpdate = False
        self.emit(Qt.SIGNAL("elementDown"))
        text = str(self.text()).split()
        if len(text) > 0:
            self.validateMacro(text[0])
        self.disableSpockCommandUpdate = True

    def upAction(self):
        self.disableSpockCommandUpdate = False
        self.emit(Qt.SIGNAL("elementUp"))
        text = str(self.text()).split()
        if len(text) > 0:
            self.validateMacro(text[0])
        self.disableSpockCommandUpdate = True

    def controlDownAction(self):
        c = self.cursorPosition()
        newValue = False
        try:
            if self.text()[c] == " " and self.text()[c - 1] == " ":
                newValue = True
        except IndexError:
            if c == 0:
                newValue = True
            elif len(self.text()) == self.cursorPosition() and self.text()[c - 1] == " ":
                newValue = True
        try:
            txt = str(self.text())
            txt = txt[:txt.find(" ", c)]
        except IndexError:
            txt = str(self.text())[:c]
        elementsNum = txt.split()

        if newValue:
            self.insert("0")
            self.currentIndex = self.getIndex(len(elementsNum))
            if not self.currentIndex.isValid():
                if len(elementsNum) > 0:
                    self.backspace()
                    return
            value = self.prevValue("")
            self.backspace()
            self.insert(value)
            self.model().setData(self.currentIndex, Qt.QVariant(value))
        else:
            self.currentIndex = self.getIndex(len(elementsNum) - 1)
            if not self.currentIndex.isValid():
                if len(elementsNum) > 1:
                    return
            value = self.prevValue(elementsNum[len(elementsNum) - 1])
            sel = self.measureSelection(self.cursorPosition())
            self.setSelection(sel[0], sel[1])
            c = c - (sel[1] - len(str(value)))
            self.insert(value)
            self.setCursorPosition(c)
            self.model().setData(self.currentIndex, Qt.QVariant(value))

    def controlUpAction(self):
        c = self.cursorPosition()
        newValue = False
        try:
            if self.text()[c] == " " and self.text()[c - 1] == " ":
                newValue = True
        except IndexError:
            if c == 0:
                newValue = True
            elif len(self.text()) == self.cursorPosition() and self.text()[c - 1] == " ":
                newValue = True
        try:
            txt = str(self.text())
            txt = txt[:txt.find(" ", c)]
        except IndexError:
            txt = str(self.text())[:c]
        elementsNum = txt.split()

        if newValue:
            self.insert("0")
            self.currentIndex = self.getIndex(len(elementsNum))
            if not self.currentIndex.isValid():
                if len(elementsNum) > 0:
                    self.backspace()
                    return
            value = self.nextValue("")
            self.backspace()
            self.insert(value)
            self.model().setData(self.currentIndex, Qt.QVariant(value))
        else:
            self.currentIndex = self.getIndex(len(elementsNum) - 1)
            if not self.currentIndex.isValid():
                if len(elementsNum) > 1:
                    return
            value = self.nextValue(elementsNum[len(elementsNum) - 1])
            sel = self.measureSelection(self.cursorPosition())
            self.setSelection(sel[0], sel[1])
            c = c - (sel[1] - len(str(value)))
            self.insert(value)
            self.setCursorPosition(c)
            self.model().setData(self.currentIndex, Qt.QVariant(value))

    def getParamItems(self, index):
        #Returns list of items that can be chosen for the node corresponding to the given index. Used by {next,prev}Value methods

        node = self.model().nodeFromIndex(index)
        if isinstance(node, macro.MacroNode):
            return None
        type = node.type()
        ms = self.getParentModelObj()
        items = ms.getElementsWithInterface(type).keys()
        return items, type

    def nextValue(self, current):
        current = str(current)
        if self.currentIndex.isValid():
            items, type = self.getParamItems(self.currentIndex)
            items = sorted(items)
        else:
            items = self.getParentModelObj().getMacroStrList()
            items = sorted(items)
            type = "Macro"

        if type == "Float":
            value = float(current) + 0.1
        elif type == "Integer":
            value = int(current) + 1
        elif type == "Boolean":
            value = True
        else:
            try:
                textindex = items.index(current)
                value = items[textindex - 1]
            except:
                tmpitems = [s for s in items if s.startswith(current)]
                if len(tmpitems) > 0:
                    value = tmpitems[0]
                else:
                    value = items[0]
        return str(value)

    def prevValue(self, current):
        current = str(current)
        if self.currentIndex.isValid():
            items, type = self.getParamItems(self.currentIndex)
            items = sorted(items)
        else:
            items = self.getParentModelObj().getMacroStrList()
            items = sorted(items)
            type = "Macro"

        if type == "Float":
            value = float(current) - 0.1
        elif type == "Integer":
            value = int(current) - 1
        elif type == "Boolean":
            value = True
        else:
            try:
                textindex = items.index(current)
                value = items[textindex + 1]
            except:
                tmpitems = [s for s in items if s.startswith(current)]
                if len(tmpitems) > 0:
                    value = tmpitems[0]
                else:
                    value = items[0]
        return str(value)

    def updateMacroEditor(self, macroName):
        #I had to make the macroname lowered as macros in comboBox (with macros), has names with all letter low.
        #Because of that sometimes it was not loading macros in MacroEditor
        #TO FIX
        self.emit(Qt.SIGNAL("spockComboBox"), str(macroName).lower())

    def measureSelection(self, position):
        s = str(self.text()) + " "
        try:
            if s[position] == " ":
                position -= 1
        except IndexError:
            position -= 1
        end = s.find(' ', position)
        beg = s.rfind(' ', 0, position + 1)
        if end == -1:
            end = s.length() - 1
        return beg + 1, end - beg - 1  #returns the start and length of the value

    def focusInEvent(self, event):
        self.disableSpockCommandUpdate = True
        Qt.QLineEdit.focusInEvent(self, event)

    def focusOutEvent(self, event):
        self.disableSpockCommandUpdate = False
        Qt.QLineEdit.focusOutEvent(self, event)

class TaurusMacroExecutorWidget(TaurusWidget):

    def __init__(self, parent=None, designMode=False):
        TaurusWidget.__init__(self, parent, designMode)
        self.setObjectName(self.__class__.__name__)

        self._doorName = ""
        self._macroId = None
        self.setLayout(Qt.QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        self.addToFavouritesAction = Qt.QAction(getThemeIcon("software-update-available"), "Add to favourites", self)
        self.connect(self.addToFavouritesAction, Qt.SIGNAL("triggered()"), self.onAddToFavourites)
        self.addToFavouritesAction.setToolTip("Add to favourites")
        self.stopMacroAction = Qt.QAction(getIcon(":/actions/media_playback_stop.svg"), "Stop macro", self)
        self.connect(self.stopMacroAction, Qt.SIGNAL("triggered()"), self.onStopMacro)
        self.stopMacroAction.setToolTip("Stop macro")
        self.pauseMacroAction = Qt.QAction(getIcon(":/actions/media_playback_pause.svg"), "Pause macro", self)
        self.connect(self.pauseMacroAction, Qt.SIGNAL("triggered()"), self.onPauseMacro)
        self.pauseMacroAction.setToolTip("Pause macro")
        self.playMacroAction = Qt.QAction(getIcon(":/actions/media_playback_start.svg"), "Start macro", self)
        self.connect(self.playMacroAction, Qt.SIGNAL("triggered()"), self.onPlayMacro)
        self.playMacroAction.setToolTip("Start macro")
        actionsLayout = Qt.QHBoxLayout()
        actionsLayout.setContentsMargins(0, 0, 0, 0)
        addToFavouritsButton = Qt.QToolButton()
        addToFavouritsButton.setDefaultAction(self.addToFavouritesAction)
        self.addToFavouritesAction.setEnabled(False)
        actionsLayout.addWidget(addToFavouritsButton)

        self.macroComboBox = MacroComboBox(self)
        self.macroComboBox.setUseParentModel(True)
        self.macroComboBox.setModelColumn(0)
        actionsLayout.addWidget(self.macroComboBox)
        stopMacroButton = Qt.QToolButton()
        stopMacroButton.setDefaultAction(self.stopMacroAction)
        actionsLayout.addWidget(stopMacroButton)
        pauseMacroButton = Qt.QToolButton()
        pauseMacroButton.setDefaultAction(self.pauseMacroAction)
        actionsLayout.addWidget(pauseMacroButton)
        self.playMacroButton = Qt.QToolButton()
        self.playMacroButton.setDefaultAction(self.playMacroAction)
        actionsLayout.addWidget(self.playMacroButton)
        self.disableControlActions()
        self.doorStateLed = TaurusLed(self)
        actionsLayout.addWidget(self.doorStateLed)
        self.layout().addLayout(actionsLayout)

        splitter = Qt.QSplitter(self)
        self.layout().addWidget(splitter)
        splitter.setOrientation(Qt.Qt.Vertical)

        self._paramEditorModel = ParamEditorModel()
        self.stackedWidget = Qt.QStackedWidget()
        self.standardMacroParametersEditor = StandardMacroParametersEditor(self.stackedWidget)
        self.stackedWidget.addWidget(self.standardMacroParametersEditor)
        self.customMacroParametersEditor = None
        splitter.addWidget(self.stackedWidget)

        self._favouritesBuffer = None
        self.favouritesMacrosEditor = FavouritesMacrosEditor(self)
        self.registerConfigDelegate(self.favouritesMacrosEditor)
        self.favouritesMacrosEditor.setUseParentModel(True)
        self.favouritesMacrosEditor.setFocusPolicy(Qt.Qt.NoFocus)

        self._historyBuffer = None
        self.historyMacrosViewer = HistoryMacrosViewer(self)
        self.registerConfigDelegate(self.historyMacrosViewer)
        self.historyMacrosViewer.setUseParentModel(True)
        self.historyMacrosViewer.setFocusPolicy(Qt.Qt.NoFocus)

        self.tabMacroListsWidget = Qt.QTabWidget(self)
        self.tabMacroListsWidget.addTab(self.favouritesMacrosEditor, "Favourite list")
        self.tabMacroListsWidget.addTab(self.historyMacrosViewer, "History Viewer")
        splitter.addWidget(self.tabMacroListsWidget)

        self._isHistoryMacro = False
        self.macroProgressBar = MacroProgressBar(self)
        self.layout().addWidget(self.macroProgressBar)

        #spockCommandLabel = Qt.QLabel("Spock command:", self)
        #spockCommandLabel.setFont(Qt.QFont("Courier",9))
        self.spockCommand = SpockCommandWidget("Spock", self)
        self.spockCommand.setSizePolicy(Qt.QSizePolicy.Expanding, Qt.QSizePolicy.Minimum)
        self.spockCommand.setUseParentModel(True)
        spockCommandLayout = Qt.QHBoxLayout()
        spockCommandLayout.setContentsMargins(0, 0, 0, 0)
        #spockCommandLayout.addWidget(spockCommandLabel)
        spockCommandLayout.addWidget(self.spockCommand)
        self.layout().addLayout(spockCommandLayout)
        self.connect(self.macroComboBox, Qt.SIGNAL("currentIndexChanged(QString)"), self.onMacroComboBoxChanged)
        self.connect(self.favouritesMacrosEditor.list, Qt.SIGNAL("favouriteSelected"), self.onFavouriteSelected)
        self.connect(self.historyMacrosViewer.list, Qt.SIGNAL("historySelected"), self.onHistorySelected)

        self.connect(self.spockCommand, Qt.SIGNAL("pressedReturn"), self.onPlayMacro)
        self.connect(self.spockCommand, Qt.SIGNAL("spockComboBox"), self.setComboBoxItem)
        self.connect(self.spockCommand, Qt.SIGNAL("elementUp"), self.setHistoryUp)
        self.connect(self.spockCommand, Qt.SIGNAL("elementDown"), self.setHistoryDown)
        self.connect(self.spockCommand, Qt.SIGNAL("setHistoryFocus"), self.setHistoryFocus)
        self.connect(self.spockCommand, Qt.SIGNAL("expandTree"), self.standardMacroParametersEditor.tree.expandAll)

    def macroId(self):
        return self._macroId

    def contextMenuEvent(self, event):
        menu = Qt.QMenu()
        action = menu.addAction(getThemeIcon("view-refresh"), "Check door state", self.checkDoorState)
        menu.exec_(event.globalPos())

    def checkDoorState(self):
        door = Device(self.doorName())
        doorState = door.state()
        if doorState == PyTango.DevState.RUNNING:
            self.playMacroAction.setEnabled(False)
            self.pauseMacroAction.setEnabled(True)
            self.stopMacroAction.setEnabled(True)
        elif doorState == PyTango.DevState.ON or doorState == PyTango.DevState.ALARM:
            self.playMacroAction.setEnabled(True)
            self.pauseMacroAction.setEnabled(False)
            self.stopMacroAction.setEnabled(False)
        elif doorState == PyTango.DevState.STANDBY:
            self.playMacroAction.setEnabled(True)
            self.pauseMacroAction.setEnabled(False)
            self.stopMacroAction.setEnabled(True)

    def setMacroId(self, macroId):
        self._macroId = macroId

    def doorName(self):
        return self._doorName

    def setDoorName(self, doorName):
        self._doorName = doorName

    def setFavouritesBuffer(self, favouritesMacro):
        self._favouritesBuffer = favouritesMacro

    #History Widget
    def setHistoryUp(self):
        self.setHistoryFocus()
        self.historyMacrosViewer.listElementUp()

    def setHistoryDown(self):
        self.setHistoryFocus()
        self.historyMacrosViewer.listElementDown()

    def setHistoryFocus(self):
        self.tabMacroListsWidget.setCurrentWidget(self.historyMacrosViewer)
        #self.historyMacrosViewer.setFocus()

    def historyBuffer(self):
        return self._historyBuffer

    def setHistoryBuffer(self, favouritesMacro):
        self._historyBuffer = favouritesMacro

    def favouritesBuffer(self):
        return self._favouritesBuffer

    def paramEditorModel(self):
        return self._paramEditorModel

    def setParamEditorModel(self, paramEditorModel):
        self._paramEditorModel = paramEditorModel

    def setComboBoxItem(self, macroName):
        self.macroComboBox.selectMacro(macroName)

    def onMacroComboBoxChanged(self, macroName):
        macroName = str(macroName)
        if macroName == "":
            macroName, macroNode = None, None
#            macroNode = macro.MacroNode(name="")
            self.playMacroAction.setEnabled(False)
            self.addToFavouritesAction.setEnabled(False)
        else:
            if self._isHistoryMacro:
                macroNode = self.historyBuffer()
                self.setHistoryBuffer(None)
                self.favouritesMacrosEditor.list.clearSelection()
            else:
                macroNode = self.favouritesBuffer()
                self.setFavouritesBuffer(None)
                self.historyMacrosViewer.list.clearSelection()
            self._isHistoryMacro = False

            if macroNode is None:
                macroNode = self.getModelObj().getMacroNodeObj(macroName)

            self.playMacroAction.setEnabled(True)
            self.addToFavouritesAction.setEnabled(True)

        self.paramEditorModel().setRoot(macroNode)
        self.spockCommand.setModel(self.paramEditorModel())
        if self.stackedWidget.count() == 2:
            self.stackedWidget.removeWidget(self.customMacroParametersEditor)
            self.customMacroParametersEditor.setParent(None)
        self.customMacroParametersEditor = ParamEditorManager().getMacroEditor(macroName, self.stackedWidget)
        if self.customMacroParametersEditor:
            self.customMacroParametersEditor.setModel(self.paramEditorModel())
            self.stackedWidget.addWidget(self.customMacroParametersEditor)
            self.stackedWidget.setCurrentWidget(self.customMacroParametersEditor)
        else:
            self.standardMacroParametersEditor.setModel(self.paramEditorModel())

        self.emit(Qt.SIGNAL("macroNameChanged"), macroName)

    def onFavouriteSelected(self, macroNode):
        self.setFavouritesBuffer(macroNode)
        name = ""
        if not macroNode is None:
            name = macroNode.name()
        self._isHistoryMacro = False
        self.macroComboBox.selectMacro(name)

    def onHistorySelected(self, macroNode):
        self.setHistoryBuffer(macroNode)
        name = ""
        if not macroNode is None:
            name = macroNode.name()
        self._isHistoryMacro = True
        self.macroComboBox.selectMacro(name)

    def onAddToFavourites(self):
        self.favouritesMacrosEditor.addMacro(deepcopy(self.paramEditorModel().root()))

    def addToHistory(self):
        self.historyMacrosViewer.addMacro(deepcopy(self.paramEditorModel().root()))

    def onDoorChanged(self, doorName):
        self.setDoorName(doorName)
        if self.doorName() == "":
            self.doorStateLed.setModel(None)
            return
        self.doorStateLed.setModel(self.doorName() + "/State")
        doorState = Device(doorName).state()
        if doorState == PyTango.DevState.ON:
            self.playMacroAction.setText("Start macro")
            self.playMacroAction.setToolTip("Start macro")
        elif doorState == PyTango.DevState.STANDBY:
            self.playMacroAction.setText("Resume macro")
            self.playMacroAction.setToolTip("Resume macro")

    def onPlayMacro(self):
        door = Device(self.doorName())
        doorState = door.state()
        if doorState == PyTango.DevState.ON or doorState == PyTango.DevState.ALARM:
            paramEditorModel = self.paramEditorModel()
            macroNode = paramEditorModel.root()
            id = macroNode.assignId()
            self.setMacroId(id)
            params, alerts = macroNode.toRun()
            xmlString = paramEditorModel.toXmlString()
            if len(alerts) > 0:
                Qt.QMessageBox.warning(self, "Macro parameters warning", alerts)
                return
            door.runMacro(xmlString)
            self.addToHistory()
#            door.runMacro(str(macroNode.name()), params)
        elif doorState == PyTango.DevState.STANDBY:
            door.command_inout("ResumeMacro")
        else:
            Qt.QMessageBox.warning(self, "Error while starting/resuming macro",
                                   "It was not possible to start/resume macro, because state of the door was different than ON/STANDBY")

    def onStopMacro(self):
        door = Device(self.doorName())
        doorState = door.state()

        if doorState in (PyTango.DevState.RUNNING, PyTango.DevState.STANDBY):
            door.command_inout("StopMacro")
        else:
            Qt.QMessageBox.warning(self, "Error while stopping macro",
                                   "It was not possible to stop macro, because state of the door was different than RUNNING or STANDBY")

    def onPauseMacro(self):
        door = Device(self.doorName())
        doorState = door.state()

        if doorState == PyTango.DevState.RUNNING:
            door.command_inout("PauseMacro")
        else:
            Qt.QMessageBox.warning(self, "Error while pausing macro",
                                   "It was not possible to pause macro, because state of the door was different than RUNNING")

    def onMacroStatusUpdated(self, data):
        macro = data[0]
        if macro is None: return
        data = data[1][0]
        state, range, step, id = data["state"], data["range"], data["step"], data["id"]
        if id is None: return
        id = int(id)
        if id != self.macroId(): return
        macroName = macro.name
        shortMessage = ""
        if state == "start":
            self.emit(Qt.SIGNAL("macroStarted"), "DoorOutput")
            self.macroProgressBar.setRange(range[0], range[1])
            self.playMacroAction.setEnabled(False)
            self.pauseMacroAction.setEnabled(True)
            self.stopMacroAction.setEnabled(True)
            self.emit(Qt.SIGNAL("plotablesFilterChanged"), None)
            self.emit(Qt.SIGNAL("plotablesFilterChanged"), standardPlotablesFilter)
            shortMessage = "Macro %s started." % macroName
        elif state == "pause":
            self.playMacroAction.setText("Resume macro")
            self.playMacroAction.setToolTip("Resume macro")
            self.playMacroAction.setEnabled(True)
            self.pauseMacroAction.setEnabled(False)
            shortMessage = "Macro %s paused." % macroName
        elif state == "resume":
            self.playMacroAction.setText("Start macro")
            self.playMacroAction.setToolTip("Start macro")
            self.playMacroAction.setEnabled(False)
            self.pauseMacroAction.setEnabled(True)
            shortMessage = "Macro %s resumed." % macroName
        elif state == "stop" or state == "finish":
            self.playMacroAction.setEnabled(True)
            self.pauseMacroAction.setEnabled(False)
            self.stopMacroAction.setEnabled(False)
            shortMessage = "Macro %s finished." % macroName
        elif state == "exception":
            self.playMacroAction.setEnabled(True)
            self.pauseMacroAction.setEnabled(False)
            self.stopMacroAction.setEnabled(False)
            shortMessage = "Macro %s error." % macroName
            exc_value, exc_stack = data['exc_value'], data['exc_stack']
            exceptionDialog = TaurusMessageBox(MacroRunException, exc_value, exc_stack)
            exceptionDialog.exec_()
        elif state == "abort":
            self.playMacroAction.setText("Start macro")
            self.playMacroAction.setToolTip("Start macro")
            self.playMacroAction.setEnabled(True)
            self.pauseMacroAction.setEnabled(False)
            self.stopMacroAction.setEnabled(False)
            shortMessage = "Macro %s stopped." % macroName
        elif state == "step":
            shortMessage = "Macro %s at %d %% of progress." % (macroName, step)
        self.emit(Qt.SIGNAL("shortMessageEmitted"), shortMessage)
        self.macroProgressBar.setValue(step)

    def disableControlActions(self):
        self.pauseMacroAction.setEnabled(False)
        self.stopMacroAction.setEnabled(False)
        self.playMacroAction.setEnabled(False)

    def setModel(self, model):
        oldModelObj = self.getModelObj()
        if oldModelObj is not None:
            self.disconnect(oldModelObj, Qt.SIGNAL("macrosUpdated"), self.macroComboBox.onMacrosUpdated)
        TaurusWidget.setModel(self, model)
        newModelObj = self.getModelObj()
        self.connect(newModelObj, Qt.SIGNAL("macrosUpdated"), self.macroComboBox.onMacrosUpdated)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return {'container': False,
                'group': 'Taurus Sardana',
                'module': 'taurus.qt.qtgui.extra_macroexecutor',
                'icon': ':/designer/frame.png'}


class TaurusMacroExecutor(MacroExecutionWindow):

    def __init__(self, parent=None, designMode=False):
        MacroExecutionWindow.__init__(self, parent, designMode)

    def initComponents(self):
        self.taurusMacroExecutorWidget = TaurusMacroExecutorWidget(self)
        self.registerConfigDelegate(self.taurusMacroExecutorWidget)
        self.taurusMacroExecutorWidget.setUseParentModel(True)
        self.setCentralWidget(self.taurusMacroExecutorWidget)
        self.connect(self.taurusMacroExecutorWidget, Qt.SIGNAL('shortMessageEmitted'), self.onShortMessage)
        self.statusBar().showMessage("MacroExecutor ready")

    def setCustomMacroEditorPaths(self, customMacroEditorPaths):
        MacroExecutionWindow.setCustomMacroEditorPaths(self, customMacroEditorPaths)
        ParamEditorManager().parsePaths(customMacroEditorPaths)
        ParamEditorManager().browsePaths()

    def loadSettings(self):
        TaurusMainWindow.loadSettings(self)
        self.emit(Qt.SIGNAL("doorChanged"), self.doorName())

    def onDoorChanged(self, doorName):
        MacroExecutionWindow.onDoorChanged(self, doorName)
        if self._qDoor:
            Qt.QObject.disconnect(self._qDoor, Qt.SIGNAL("macroStatusUpdated"), self.taurusMacroExecutorWidget.onMacroStatusUpdated)
        if doorName == "": return
        self._qDoor = Device(doorName)
        Qt.QObject.connect(self._qDoor, Qt.SIGNAL("macroStatusUpdated"), self.taurusMacroExecutorWidget.onMacroStatusUpdated)
        self.taurusMacroExecutorWidget.onDoorChanged(doorName)

    @classmethod
    def getQtDesignerPluginInfo(cls):
        return None


def createMacroExecutorWidget(args):
    macroExecutor = TaurusMacroExecutorWidget()
    macroExecutor.setModelInConfig(True)
    Qt.QObject.connect(macroExecutor, Qt.SIGNAL("doorChanged"), macroExecutor.onDoorChanged)
    if len(args) == 2:
        macroExecutor.setModel(args[0])
        macroExecutor.emit(Qt.SIGNAL('doorChanged'), args[1])
    return macroExecutor

def createMacroExecutor(args):
    macroExecutor = TaurusMacroExecutor()
    macroExecutor.setModelInConfig(True)
    Qt.QObject.connect(macroExecutor, Qt.SIGNAL("doorChanged"), macroExecutor.onDoorChanged)
    if len(args) == 2:
        macroExecutor.setModel(args[0])
        macroExecutor.emit(Qt.SIGNAL('doorChanged'), args[1])
    macroExecutor.loadSettings()
    return macroExecutor

def main():
    from taurus.qt.qtgui.application import TaurusApplication
    import taurus

    app = TaurusApplication(sys.argv, app_version=taurus.Release.version)
    args = app.get_command_line_args()

    app.setOrganizationName("Taurus")
    app.setApplicationName("macroexecutor")
    macroExecutor = createMacroExecutor(args)
    macroExecutor.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Pool library. It defines the base classes
for"""

__all__ = ["PoolBaseElement"]

__docformat__ = 'restructuredtext'

import weakref
import threading

from taurus.core.util.lock import TaurusLock

from sardana import State
from sardana.sardanaevent import EventType
from sardana.pool.poolobject import PoolObject


class PoolBaseElement(PoolObject):
    """A Pool object that besides the name, reference to the pool, ID, full_name
    and user_full_name has:
       
       - _simulation_mode : boolean telling if in simulation mode
       - _state : element state
       - _status : element status"""

    def __init__(self, **kwargs):
        self._simulation_mode = False
        self._state = None
        self._state_event = None
        self._status = None
        self._status_event = None
        self._action_cache = None
        self._aborted = False
        self._stopped = False

        lock_name = kwargs['name'] + "Lock"

        # A lock for high level operations: monitoring, motion or acquisition
        self._lock = TaurusLock(name=lock_name, lock=threading.RLock())

        # The operation context in which the element is involved
        self._operation = None

        # The :class:`PoolAction` in which element is involved
        self._pool_action = None

        super(PoolBaseElement, self).__init__(**kwargs)

    def __enter__(self):
        self.lock()

    def __exit__(self, exc_type, exc_value, traceback):
        self.unlock()
        return False

    def lock(self, blocking=True):
        """Acquires the this element lock
        
        :param blocking:
            whether or not to block if lock is already acquired [default: True]
        :type blocking: bool"""
        ret = self._lock.acquire(blocking)
        return ret

    def unlock(self):
        ret = self._lock.release()
        return ret

    def get_action_cache(self):
        """Returns the internal action cache object"""
        return self._action_cache

    def serialize(self, *args, **kwargs):
        ret = PoolObject.serialize(self, *args, **kwargs)
        return ret

    # --------------------------------------------------------------------------
    # simulation mode
    # --------------------------------------------------------------------------

    def get_simulation_mode(self, cache=True, propagate=1):
        """Returns the simulation mode for this object.
        
        :param cache: not used [default: True]
        :type cache: bool
        :param propagate: [default: 1]
        :type propagate: int
        :return: the current simulation mode
        :rtype: bool"""
        return self._simulation_mode

    def set_simulation_mode(self, simulation_mode, propagate=1):
        self._simulation_mode = simulation_mode
        if not propagate:
            return
        if simulation_mode == self._simulation_mode:
            # current state is equal to last state_event. Skip event
            return
        self.fire_event(EventType("simulation_mode", priority=propagate),
                        simulation_mode)

    def put_simulation_mode(self, simulation_mode):
        self._simulation_mode = simulation_mode

    simulation_mode = property(get_simulation_mode, set_simulation_mode,
                               doc="element simulation mode")

    # --------------------------------------------------------------------------
    # state
    # --------------------------------------------------------------------------

    def get_state(self, cache=True, propagate=1):
        """Returns the state for this object. If cache is True (default) it
        returns the current state stored in cache (it will force an update if
        cache is empty). If propagate > 0 and if the state changed since last
        read, it will propagate the state event to all listeners.
        
        :param cache:
            tells if return value from local cache or update from HW read
            [default: True]
        :type cache: bool
        :param propagate:
            if > 0 propagates the event in case it changed since last HW read.
            Values bigger that mean the event if sent should be a priority event
            [default: 1]
        :type propagate: int
        :return: the current object state
        :rtype: :obj:`sardana.State`"""
        if not cache or self._state is None:
            state_info = self.read_state_info()
            self._set_state_info(state_info, propagate=propagate)
        return self._state

    def inspect_state(self):
        """Looks at the current cached value of state

        :return: the current object state
        :rtype: :obj:`sardana.State`"""
        return self._state

    def set_state(self, state, propagate=1):
        self._set_state(state, propagate=propagate)

    def _set_state(self, state, propagate=1):
        self._state = state
        if not propagate:
            return
        if state == self._state_event:
            # current state is equal to last state_event. Skip event
            return
        self._state_event = state
        self.fire_event(EventType("state", priority=propagate), state)

    def put_state(self, state):
        self._state = state

    state = property(get_state, set_state, doc="element state")

    # --------------------------------------------------------------------------
    # status
    # --------------------------------------------------------------------------

    def inspect_status(self):
        """Looks at the current cached value of status

        :return: the current object status
        :rtype: str"""
        return self._status

    def get_status(self, cache=True, propagate=1):
        """Returns the status for this object. If cache is True (default) it
        returns the current status stored in cache (it will force an update if
        cache is empty). If propagate > 0 and if the status changed since last
        read, it will propagate the status event to all listeners.
        
        :param cache:
            tells if return value from local cache or update from HW read
            [default: True]
        :type cache: bool
        :param propagate:
            if > 0 propagates the event in case it changed since last HW read.
            Values bigger that mean the event if sent should be a priority event
            [default: 1]
        :type propagate: int
        :return: the current object status
        :rtype: str"""
        if not cache or self._status is None:
            state_info = self.read_state_info()
            self._set_state_info(state_info, propagate=propagate)
        return self._status

    def set_status(self, status, propagate=1):
        self._set_status(status, propagate=propagate)

    def _set_status(self, status, propagate=1):
        self._status = status
        if not propagate:
            return
        s_evt = self._status_event
        if s_evt is not None and len(status) == len(s_evt) and status == s_evt:
            # current status is equal to last status_event. Skip event
            return
        self._status_event = status
        self.fire_event(EventType("status", priority=propagate), status)

    def put_status(self, status):
        self._status = status

    status = property(get_status, set_status, doc="element status")

    # --------------------------------------------------------------------------
    # state information
    # --------------------------------------------------------------------------

    _STD_STATUS = "{name} is {state}\n{ctrl_status}"
    def calculate_state_info(self, status_info=None):
        """Transforms the given state information. This specific base
        implementation transforms the given state,status tuple into a
        state, new_status tuple where new_status is "*self.name* is *state*
        plus the given status.
        It is assumed that the given status comes directly from the controller
        status information.
        
        :param status_info:
            given status information [default: None, meaning use current state status.
        :type status_info: tuple<State, str>
        :return: a transformed state information
        :rtype: tuple<State, str>"""
        if status_info is None:
            status_info = self._state, self._status
        state, status = status_info
        state_str = State[state]
        new_status = self._STD_STATUS.format(name=self.name, state=state_str,
                                             ctrl_status=status)
        return status_info[0], new_status

    def set_state_info(self, state_info, propagate=1):
        self._set_state_info(state_info, propagate=propagate)

    def _set_state_info(self, state_info, propagate=1):
        state_info = self.calculate_state_info(state_info)
        state, status = state_info[:2]
        self._set_status(status, propagate=propagate)
        self._set_state(state, propagate=propagate)

    def read_state_info(self):
        action_cache = self.get_action_cache()
        ctrl_state_info = action_cache.read_state_info(serial=True)[self]
        return self._from_ctrl_state_info(ctrl_state_info)

    def put_state_info(self, state_info):
        self.set_state_info(state_info, propagate=0)

    def _from_ctrl_state_info(self, state_info):
        try:
            state_str = State.whatis(state_info)
            return int(state_info), "{0} is in {1}".format(self.name, state_str)
        except KeyError:
            pass
        state_info, _ = state_info
        state, status = state_info[:2]
        state = int(state)
        return state, status

    # --------------------------------------------------------------------------
    # default attribute
    # --------------------------------------------------------------------------

    def get_default_attribute(self):
        return NotImplementedError("%s doesn't have default attribute" % self.__class__.__name__)

    # --------------------------------------------------------------------------
    # default acquisition channel name
    # --------------------------------------------------------------------------

    def get_default_acquisition_channel(self):
        return self.get_default_attribute().name

    # --------------------------------------------------------------------------
    # stop
    # --------------------------------------------------------------------------

    def stop(self):
        self._stopped = True

    def was_stopped(self):
        return self._stopped

    # --------------------------------------------------------------------------
    # abort
    # --------------------------------------------------------------------------

    def abort(self):
        self._aborted = True

    def was_aborted(self):
        return self._aborted

    # --------------------------------------------------------------------------
    # interrupted
    # --------------------------------------------------------------------------

    def was_interrupted(self):
        """Tells if action ended by an abort or stop"""
        return self.was_aborted() or self.was_stopped()

    # --------------------------------------------------------------------------
    # involved in an operation
    # --------------------------------------------------------------------------

    def is_action_running(self):
        """Determines if the element action is running or not."""
        return self.get_action_cache().is_running()

    def is_in_operation(self):
        """Returns True if this element is involved in any operation"""
        return self.get_operation() is not None

    def is_in_local_operation(self):
        return self.get_operation() == self.get_action_cache()

    def get_operation(self):
        return self._operation

    def set_operation(self, operation):
        if self.is_in_operation() and operation is not None:
            raise Exception("%s is already involved in an operation"
                            % self.name)
        if operation is not None:
            self._aborted = False
            self._stopped = False
        self._operation = operation

    def clear_operation(self):
        return self.set_operation(None)


class PoolElement(PoolBaseElement):
    """A Pool element is an Pool object which is controlled by a controller.
       Therefore it contains a _ctrl_id and a _axis (the id of the element in
       the controller)."""

    def __init__(self, **kwargs):
        ctrl = kwargs.pop('ctrl')
        self._ctrl = weakref.ref(ctrl)
        self._axis = kwargs.pop('axis')
        self._ctrl_id = ctrl.get_id()
        try:
            instrument = kwargs.pop('instrument')
            self.set_instrument(instrument)
        except KeyError:
            self._instrument = None
        super(PoolElement, self).__init__(**kwargs)

    def serialize(self, *args, **kwargs):
        kwargs = PoolBaseElement.serialize(self, *args, **kwargs)
        kwargs['controller'] = self.controller.full_name
        kwargs['unit'] = '0'  #TODO: hardcoded unit to 0
        kwargs['axis'] = self.axis
        if self.instrument is not None:
            kwargs['instrument'] = self.instrument.full_name
        else:
            kwargs['instrument'] = None
        kwargs['source'] = self.get_source()
        return kwargs

    def get_parent(self):
        return self.get_controller()

    def get_controller(self):
        if self._ctrl is None:
            return None
        return self._ctrl()

    def get_controller_id(self):
        return self._ctrl_id

    def get_axis(self):
        return self._axis

    def set_action_cache(self, action_cache):
        self._action_cache = action_cache
        action_cache.add_element(self)

    def get_source(self):
        return "{0}/{1}".format(self.full_name, self.get_default_acquisition_channel())

    # --------------------------------------------------------------------------
    # instrument
    # --------------------------------------------------------------------------

    def get_instrument(self):
        if self._instrument is None:
            return None
        return self._instrument()

    def set_instrument(self, instrument, propagate=1):
        self._set_instrument(instrument, propagate=propagate)

    def _set_instrument(self, instrument, propagate=1):
        if self._instrument is not None:
            self._instrument().remove_element(self)
        new_instrument_name = ""
        if instrument is None:
            self._instrument = None
        else:
            self._instrument = weakref.ref(instrument)
            new_instrument_name = instrument.full_name
            instrument.add_element(self)
        if not propagate:
            return
        self.fire_event(EventType("instrument", priority=propagate),
                        new_instrument_name)

    # --------------------------------------------------------------------------
    # stop
    # --------------------------------------------------------------------------

    def stop(self):
        self.info("Stop!")
        PoolBaseElement.stop(self)
        self.controller.stop_one(self.axis)

    # --------------------------------------------------------------------------
    # abort
    # --------------------------------------------------------------------------

    def abort(self):
        self.info("Abort!")
        PoolBaseElement.abort(self)
        self.controller.abort_one(self.axis)

    def get_par(self, name):
        return self.controller.get_axis_par(self.axis, name)

    def set_par(self, name, value):
        return self.controller.set_axis_par(self.axis, name, value)

    def get_extra_par(self, name):
        return self.controller.get_axis_attr(self.axis, name)

    def set_extra_par(self, name, value):
        return self.controller.set_axis_attr(self.axis, name, value)

    axis = property(get_axis, doc="element axis")
    controller = property(get_controller, doc="element controller")
    controller_id = property(get_controller_id, doc="element controller id")
    instrument = property(get_instrument, set_instrument,
                          doc="element instrument")



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""
__init__.py: 
"""

from motor import TaurusMotorH, TaurusMotorH2, TaurusMotorV, TaurusMotorV2
from poolmotor import PoolMotorSlim, LabelWidgetDragsDeviceAndAttribute
from poolmotor import PoolMotorTV, PoolMotorTVLabelWidget, PoolMotorTVReadWidget, PoolMotorTVWriteWidget, PoolMotorTVUnitsWidget
from poolmotor import PoolMotor
from poolchannel import PoolChannel, PoolChannelTV
from poolioregister import PoolIORegisterTV, PoolIORegisterReadWidget, PoolIORegisterWriteWidget, PoolIORegister, PoolIORegisterButtons



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Pool libray. It defines the base classes
for moveable elements"""

__all__ = ["PoolMoveable"]

__docformat__ = 'restructuredtext'


class PoolMoveable(object):

    def get_size(self):
        raise RuntimeError("Must be implemented in subclass")

    def calc_move(self, positions, ctrl_map, trust=False):
        raise RuntimeError("Must be implemented in subclass")

    def set_value(self, v, propagate=True):
        raise RuntimeError("Must be implemented in subclass")

    def get_value(self, cache=True):
        raise RuntimeError("Must be implemented in subclass")



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains the function to access sardana thread pool"""

from __future__ import with_statement
from __future__ import absolute_import

__all__ = ["get_thread_pool"]

__docformat__ = 'restructuredtext'

import threading

from taurus.core.util.threadpool import ThreadPool

__thread_pool_lock = threading.Lock()
__thread_pool = None


def get_thread_pool():
    """Returns the global pool of threads for Sardana
    
    :return: the global pool of threads object
    :rtype: taurus.core.util.ThreadPool"""

    global __thread_pool
    global __thread_pool_lock
    with __thread_pool_lock:
        if __thread_pool is None:
            __thread_pool = ThreadPool(name="SardanaTP", Psize=10)
        return __thread_pool
